/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class AccountList function definition 
 *
 * 2005-08-18
 * Olivier Delannoy
 */
#include "AccountList.hh"
#include <LoggerFactory.hh>
#include <XMLSerializer.hh> 
AccountList::AccountList(void)
{
    
}
AccountList::~AccountList(void) 
{
    
}
bool AccountList::insert(const Yml::Core::string& login, const Yml::Core::string& password, 
                         const Yml::Core::string& email, const Yml::Core::string& rights, 
                         const Yml::Core::string& fullname, const Yml::Core::string& organization)
{
    Account* acc = find(login);
    if (!acc)
    {
        Account nAcc;
        nAcc.mLogin = login;
        nAcc.mPassword = password;
        nAcc.mEmail = email;
        nAcc.mRights = rights;
        nAcc.mFullName = fullname;
        nAcc.mOrganization = organization;
        push_back(nAcc);
        return true;
    }
    return false;
}
bool AccountList::remove(const Yml::Core::string& login)
{
    for (iterator iter = begin() ; iter != end() ; ++iter)
    {
        if (iter->mLogin == login)
        {
            erase(iter);
            return true;
        }
    }
    return false;
}   
bool AccountList::update(const Yml::Core::string& login,const Yml::Core::string& password, 
                         const Yml::Core::string& email, const Yml::Core::string& rights, 
                         const Yml::Core::string& fullname, const Yml::Core::string& organization)
{
    Account* acc = find(login);
    if (acc)
    {
        if (password != "")
            acc->mPassword = password;
        if (email != "")
            acc->mEmail = email;
        if (rights != "")
            acc->mRights = rights;
        if (fullname != "")
            acc->mFullName = fullname;
        if (organization != "")
            acc->mOrganization = organization;
        return true;
    }
    return false;
}
Account* AccountList::find(const Yml::Core::string& login)
{
    Account* res = 0;
    UTIL_DEBUG("default", "Search for '" << login << "'");
    UTIL_DEBUG("default", "Registered account " << size());
    for (iterator iter = begin() ; iter != end() ; ++iter)
    {
        UTIL_DEBUG("default", login << "<=>" << iter->mLogin);
        if (iter->mLogin == login)
        {
            res = &(*iter);
            break;
        }
        
    }
    return res;
}

void AccountList::toXML(std::ostream& out) const
{
    Util::XMLSerializer stream(out);
    stream.openTag("accounts");
    for (const_iterator iter = begin() ; iter != end() ; ++iter)
    {
        stream.openTag("account")
            .attribute("login", iter->mLogin)
            .attribute("password", iter->mPassword)
            .attribute("email", iter->mEmail)
            .attribute("rights", iter->mRights)
            .attribute("fullname", iter->mFullName)
            .attribute("organization", iter->mOrganization)
            .closeTag();
    }
    stream.closeTag();
}
