/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class XMLAccountManager declaration
 *  
 * 2005-06-17
 * Olivier Delannoy
 */
#ifndef PLUGINS_ACCOUNTMANAGERS_XMLACCOUNTMANAGER_HH
#define PLUGINS_ACCOUNTMANAGERS_XMLACCOUNTMANAGER_HH 1
#include <Yml/Core/AccountManager.hh>
namespace XMLAccountManager {
    /**
     * @brief XML based Account Manager class.         
     *
     * This class use an XML file to store user information.
     * Each user has the following field : 
     * - login 
     * - password
     * - email
     * - full name 
     * - organization 
     * - rights 
     * 
     * The XML file format is as follow : 
     * @code
     * <?xml version="1.0"?>
     * <accounts>
     *   <account login="blabla" password="blabla" rights="10" email="" fullname="" organization="" />
     *   ... 
     * </accounts>
     * @endcode
     * 
     * initData contains only one value, it corresponds to the
     * file name containing the accounts information. 
     */
    class XMLAccountManager : public Yml::Core::AccountManager 
    {
    public:
        /**
         * Default Contstructor
         */
        XMLAccountManager(void);
        /**
         * Destructor
         */
        virtual ~XMLAccountManager(void);
        /**
         * initialize the AccountManager instance. 
         * @param initData contains the name of the file storing account information 
         */
        virtual bool init(const Yml::Core::string& initData);
        virtual Yml::Core::Account* load(const Yml::Core::string& login, const Yml::Core::string& password);
        virtual bool insert(Yml::Core::Account* account);
        virtual bool update(Yml::Core::Account* account);
        virtual bool remove(const Yml::Core::string& login);
    protected:
                
    private:            
        /** Store the file name */
        Yml::Core::string mFile;
    };
}

#endif

