/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class ProcessBackend function definition 
 *
 * 2007-09-22
 * Olivier Delannoy
 */
#include "DotBackend.hh"

#include <UID.hh>
#include <MD5.hh>
#include <RuntimeEnvironment.hh>
#include <LoggerFactory.hh>
#include <ConfigFactory.hh>
#include <Pack.hh>


namespace DotBackend
{

// Finished 
DotBackend::DotBackend()
    : mLastTask(0), mDest(0)
{
}
 
// Finished
DotBackend::~DotBackend()
{
    mDest->close();
    delete mDest;
}

// Finished 
void DotBackend::initImpl(const Yml::Core::string& initData)
{
    mDest = new std::ofstream("application.dot");
    assert(mDest);
    
    *mDest << "digraph application {" << std::endl;
    // Output the start of the graph 
    // Print the attribute of the start node 
    *mDest << "  node start [shape = doublecircle];" << std::endl;
    //*mDest << "  node stop  [shape = doublecircle];" << std::endl;
}

void DotBackend::cleanupImpl() 
{
    // Make the link to the stop node
    *mDest << "}" << std::endl;
    mDest->close();
    delete mDest;
}
Yml::Core::SchedulerTask* DotBackend::retrieveImpl()
{
    Yml::Core::SchedulerTask* res = 0;
    if (!mTasks.empty())
    {
        if (mLastTask) printNode(mLastTask);
        mLastTask = mTasks.front();
        mTasks.pop_front();
        res = mLastTask;
        if (res)
        {
            // Create the resulting packs 
            Util::Pack pack(res->packAdmin());    
            pack.add("trace.txt", (Util::byte*)"", 0);
            pack.add("status.txt", (Util::byte*)"SUCCESS\n", 7);
            Util::Pack packOut(res->packOutput());
            res->setStatus(Yml::Core::ExecSuccess);
        }
    }
    return res;
}

bool DotBackend::executeImpl(Yml::Core::SchedulerTask* task)
{
    if (mLastTask == 0)
        *mDest << "    start -> t" << task->index() << ";" << std::endl;
    else 
        *mDest << "    t" << mLastTask->index() << " -> t" << task->index() << ";" << std::endl;
    mTasks.push_back(task);
    mTasks.push_back(0);
    
    return true;
}





void DotBackend::printNode(Yml::Core::SchedulerTask* task)
{
    assert(task);
    UTIL_DEBUG("", "Task: " << task->component());
    const Yml::Core::uint32 pCount = task->parameters();
    *mDest << "node t" << task->index() << " [label=\"" << task->component() << "\\n";
    for(Yml::Core::uint32 i = 0 ; i < pCount ; ++i)
    {
        *mDest << task->parameter(i) << " ";
    }
    *mDest << "\"];" 
           << std::endl;
}

}

/*! 
    Plugin access function 
 */ 
extern "C" DotBackend::DotBackend* create(void)
{
    return new DotBackend::DotBackend;
}

/*! 
    Plugin instance deallocator function 
 */
extern "C" void destroy(DotBackend::DotBackend* object)
{
    delete object;
}

