/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class DotBackend declaration
 *  
 * 2007-09-22
 * Olivier Delannoy
 */
#ifndef DOTBACKEND_HH
#define DOTBACKEND_HH 1
#include <Yml/Core/Backend.hh>
#include <Yml/Core/SchedulerTask.hh>
#include <list>
#include <set>
#include <fstream> 

namespace DotBackend
{
/*! 
  \brief 
  Dot based backend.

  Dot based backend. This backend is usefull to test an
  application on a single host. Everything is made so that an
  application running on this backend works exactly the same as on a
  distributed platform using one of the other backend available 
*/
class DotBackend : public Yml::Core::Backend
{
    typedef std::list<Yml::Core::SchedulerTask*> TaskList;
public:
    DotBackend();
    ~DotBackend();
protected:        
    void initImpl(const Yml::Core::string& initData);
    void cleanupImpl();

    bool executeImpl(Yml::Core::SchedulerTask* task);
    Yml::Core::SchedulerTask* retrieveImpl(void);
private:
    void printNode(Yml::Core::SchedulerTask* task);
    
    /** Store tasks in the list */
    TaskList mTasks;
    /** Store the last task retrieved */ 
    Yml::Core::SchedulerTask* mLastTask;
    /** Destination of the dot file */ 
    std::ofstream* mDest;
};

}
#endif

