/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class OmniRPCBackend declaration
 *  
 * 2005-07-02
 * Choy Laurent
 * Olivier Delannoy
 *
 *
 */

#ifndef OMNIRPCBACKEND_HH
#define OMNIRPCBACKEND_HH 1
// YML include files
#include <Yml/Core/Backend.hh>
#include <Yml/Core/SchedulerTask.hh>
#include <Yml/Core/ExecutionCatalog.hh>
#include <Yml/Core/PluginManager.hh>

// stdlib include file
#include <cstdlib>

#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

// Util include files
#include <Config.hh>
#include <ConfigXCF.hh>
#include <ConfigFactory.hh>


#ifdef __cplusplus
extern "C" {
#endif
#include "OmniRpc.h"
#ifdef __cplusplus
}
#endif

namespace OmniRPCBackend
{       
/** 
 * @brief The OmniRPC backend class for Yml 
 *
 * This class implement the omnirpc backend interface for the 
 * YML Framework. 
 * It's the glue between YML and omnirpc workers
 */ 

class OmniRPCBackend: public Yml::Core::Backend {
public:
            
    /**
     * Constructor
     */
    OmniRPCBackend();
    /**
     * Destructor
     */
    ~OmniRPCBackend();        
protected:
    /**
     * It initialyzes the OmniRPC platform
     * @param initData currently unused
     */
    void initImpl(const Yml::Core::string &initData);
    /**
     * It sends a task to a remote node by means of an asynchroneous OmniRPC call
     * @param task the task that must be solved by the remote node
     */
    bool executeImpl(Yml::Core::SchedulerTask* theTask);
    /**
     * It retrieves a vector of finished tasks
     * @param sizeVector size of the returned vector of finished tasks
     */
    //std::vector<Yml::Core::SchedulerTask*> Plugins::OmniRPCBackend::OmniRPCBackend::retrieve(Yml::Core::uint16 &sizeVector);
            
    /**
     * It retrieves the result of a finished task             
     */
    Yml::Core::SchedulerTask * retrieveImpl (void);
    
private:
    bool initializeOmniRPC();
    void finalizeOmniRPC();
    
    Yml::Core::SchedulerTask * internalRetrieve (void);
    int MAX_NUMBER_REQUEST;        
    bool mMustInit;            
            
    OmniRpcRequest* req;            
    int* flagReq;
    int* taskStatus;
    Yml::Core::SchedulerTask** taskReq;   
    std::vector<Yml::Core::SchedulerTask*> someCompletedTasks;

    /** Store the path to the components relative to the DRServer*/
    Yml::Core::string mBinaryGroup;
    /** Store the path to the packIn dir */
    Yml::Core::string mPackInDir;
    /** Store the path to the packOut dir */
    Yml::Core::string mPackOutDir;
    /** Store the version of OmniRPC */ 
    Yml::Core::string mOmniVersion;      
    /** Store the hostfile directory of the OmniRPC client of the platform 
        (typically the backend machine)                                   
    */
    Yml::Core::string mHostFileDir;            
    /** Store wether the backend is verbose or not */
    bool mEnableDebug;
    
    
    /** Store the drserver host */
    Yml::Core::string mDRHost;
    /** Store the port number of the drserver */
    Yml::Core::string mDRPort;
};        
}

#endif 

