/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class XtremWebQuery declaration
 *  
 * 2005-10-28
 * Olivier Delannoy
 *
 *
 */
#ifndef XTREMWEBQUERY_HH
#define XTREMWEBQUERY_HH 1
#include <Yml/Core/core.hh>
namespace XtremWebBackendDB
{
    /** 
     * @brief XtremWeb client is a class used to generate SQL query
     * independantly of the XtremWeb version.
     *
     * This class is used to generate a set of SQL query depending on
     * the version of XtremWeb. Submitting a job to XtremWeb is a two
     * step operation: 
     * - Selecting the first unused works uid 
     * - Inserting a row using the previously selected uid in table
     * works.
     * 
     * On the otherside detecting the terminaison of XtremWeb is
     * also a two step operation: 
     * - Selecting all finished works (status COMPLETED) 
     * - Deleting works entry in order to increase the speed of later query. 
     *
     */
    class XtremWebQuery 
    {
    public:
        XtremWebQuery(const Yml::Core::string& version);
        
        /**
         * destructor 
         */
         ~XtremWebQuery(void);

        Yml::Core::string getUserId(const Yml::Core::string& name) const;
        Yml::Core::string getAppId(const Yml::Core::string& name) const;
        Yml::Core::string insertWork(const Yml::Core::string& uid, 
                                      const Yml::Core::string& userid, 
                                      const Yml::Core::string& appid, 
                                      const Yml::Core::string& cmdLine, 
                                      const Yml::Core::string& server) const;
        Yml::Core::string getCompletedWorks(const Yml::Core::string& uid) const;
        Yml::Core::string deleteCompletedWorks(const Yml::Core::string& uid) const;

    protected:
    private:
        Yml::Core::uint32 mVersion;
        
        
    };
  }
#endif

