/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class XtremWebBackend declaration
 *  
 * 2005-07-02
 * Olivier Delannoy
 *
 *
 */
#ifndef XTREMWEBBACKEND_HH
#define XTREMWEBBACKEND_HH 1
#include <Yml/Core/Backend.hh>
#include <Yml/Core/SchedulerTask.hh>
#include <Yml/Core/ExecutionCatalog.hh>
#include <Yml/Core/PluginManager.hh>
#include <cassert> 
#include <list>
#include <deque>
/** 
    \brief 
    This namespace contains class related to XtremWeb backend for XW > 1.5.
    
    This namespace contains class related to XtremWeb backend for XW >
    1.5. This version is working using network client library instead
    of direct connect library. 
*/

namespace XtremWebBackendNET
{
class XWClient;

/** 
 * @brief The XtremWeb backend class for Yml 
 *
 * This class implement the Backend interface for the 
 * YML Framework 
 */ 
class XtremWebBackendNET : public Yml::Core::Backend
{
public:
    /** 
     * Default Constructor 
     */
    XtremWebBackendNET(void);
    /** 
     * Destructor 
     */
    ~XtremWebBackendNET(void);
protected:
    /* Inherited doc */
    void initImpl(const Yml::Core::string& initData);
    /* Inherited doc */
    bool executeImpl(Yml::Core::SchedulerTask* task);
    /* Inherited doc */
    Yml::Core::SchedulerTask* retrieveImpl(void);      
    

private:
    void pollActiveWorks();
    

    typedef std::list<std::pair<Yml::Core::string, Yml::Core::SchedulerTask*> > TaskList;
    typedef std::deque<Yml::Core::SchedulerTask*> TaskQueue;
    
    // All of this is used while creating the work description    
    /** Store the dir for tasks */
    Yml::Core::string mWorkDescDir;
    /** Store task information */
    Yml::Core::string mWorkDescGroup;
    /** Store the application UID */ 
    Yml::Core::string mAppUID;
    
    
    
    // Client to connect to the dispatcher 
    XWClient* mClient;
    // Various queue needed to store the tasks
    /** Store all finished Task */
    TaskQueue mFinished;
    /** Store all runing tasks */
    TaskList mActive;
};
}

#endif

