/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class FileCatalog declaration
 *  
 * 2005-06-14
 * Olivier Delannoy
 *
 */
#ifndef DEFAULTDEVELOPMENTCATALOG_HH
#define DEFAULTDEVELOPMENTCATALOG_HH 1
#include <Yml/Core/DevelopmentCatalog.hh>
namespace DefaultDevelopmentCatalog 
{
/**
 * @brief Catalog implementation relying on a set of directories on the filesystem. 
 *
 * This catalog implementation relies on the filesystem to search
 * and organized component. 
 */
class DefaultDevelopmentCatalog : public Yml::Core::DevelopmentCatalog
{
public:
    /** Constructor */ 
    DefaultDevelopmentCatalog(void);
    /** Destructor */
    ~DefaultDevelopmentCatalog(void);
    /**
     * There is only one parameter in the init string of this
     * catalog driver.  It corresponds to the root of the
     * repository.
     */
    void init(const Yml::Core::string& initData);
    
protected:
    bool registerComponentAbstract(const Yml::Core::ComponentAbstract *component, bool force);
    bool registerComponentGraph(const Yml::Core::ComponentGraph *component, bool force);
    bool registerComponentImpl(const Yml::Core::ComponentImpl *component, bool force);
    
    void releaseComponentAbstract(Yml::Core::ComponentAbstract *component);
    void releaseComponentGraph(Yml::Core::ComponentGraph *component);
    void releaseComponentImpl(Yml::Core::ComponentImpl *component);
    
    const Yml::Core::ComponentAbstract* retrieveComponentAbstract(const Yml::Core::string &name);
    const Yml::Core::ComponentGraph* retrieveComponentGraph(const Yml::Core::string &name);
    const Yml::Core::ComponentImpl* retrieveComponentImpl(const Yml::Core::string &name);
    
private:
    /** store the path to the abstract components */
    Yml::Core::string mAbstractDir;
    /** store the path to the graph components */
    Yml::Core::string mGraphDir;
    /** store the path to the graph implementations */
    Yml::Core::string mImplDir;    
};    
}
#endif 

