/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Miwako Tsuji 2012
 *
 *   Miwako Tsuji
 *   tsuji@hpcs.cs.tsukuba.ac.jp
 *   Tsukuba University
 *   Tsuuba, Ibaraki JAPAN
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/
/**
 * @file
 * @brief Class XMPBinaryGenerator function definition 
 *
 */
#include "XMPBinaryGenerator.hh"

#ifdef HAVE_CONFIG_H
#include <yml_config.hh>
#endif 
#include <LoggerFactory.hh>
#include <File.hh>
#include <StringList.hh>
#include <Process.hh>
#include <ConfigFactory.hh>
#include <RuntimeEnvironment.hh>
#include <StringHelper.hh>
#include <fstream>
#include <sstream>
#include <cstdlib>
#include <Yml/Core/core.hh>


#define MAXSIZE 1024

namespace DefaultExecutionCatalog
{
XMPBinaryGenerator::XMPBinaryGenerator(const Yml::Core::string& dataDir)
: BinaryGenerator(dataDir + "XMP" + Util::RuntimeEnvironment::PATH_SEPARATOR)
{
}



bool XMPBinaryGenerator::generate(const Yml::Core::ComponentImpl& impl, const Yml::Core::ParameterList& interface, const Yml::Core::string& dest)
{
	// Create source file
	Yml::Core::string src(dest + ".c");
	Yml::Core::string idl_src(dest+".idl");
	if (createSourceCode(impl, interface, src))
	{
		if (createBinary(impl.getImplLibraries(), dest, src))
		{
			UTIL_INFO("default", "DefaultExecutionCatalog: Implementation component '" << impl.getName() << "' compilation successfull");
			if(createOmrpcIdl(impl, interface, idl_src))
			{
				if(createRex(impl.getImplLibraries(), dest, idl_src))
				{
					UTIL_INFO("default", "DefaultExecutionCatalog: Implementation component '" << impl.getName() << "' omnirpc-rex creation successfull");
					return true;
				}
			}
			else
				UTIL_ERROR("default", "DefaultExecutionCatalog: Implementation component '" << impl.getName() << "' omnirpc-rex creation failed");

		}
		else
		{
			//Util::File::remove(src);
			UTIL_ERROR("default", "DefaultExecutionCatalog: Implementation component '" << impl.getName() << "' compilation failed");
		}
	}
	else
	{
		UTIL_ERROR("default", "DefaultExecutionCatalog: Implementation component '" << impl.getName() << "' source code generation failed");
	}
	return false;

}

bool XMPBinaryGenerator::createOmrpcIdl(const Yml::Core::ComponentImpl& impl, const Yml::Core::ParameterList& interface, const Yml::Core::string& dest)
{
	// Create OmniRPC IDL
	std::ofstream out(dest.c_str());
	if (!out)
		return false;
	out << "Module " << impl.getComponentAbstractName() << ";" << std::endl << std::endl;
	out << "Globals {" << std::endl
			<< "#include <Yml/Worker/worker.h>" << std::endl
			<< "#include<limits.h>" << std::endl
			<< "#include\"xmp_header.hh\"" << std::endl
			<< "struct droperation" << std::endl
			<< "{" << std::endl
			<< "    char*          mHost;" << std::endl
			<< "    unsigned short mPort;" << std::endl
			<< "    char*          mUri;" << std::endl
			<< "    uint8          mIsPacked;" << std::endl
			<< "};" << std::endl
			<< "typedef struct droperation droperation_t;" << std::endl
			<< "struct worker" << std::endl
			<< "{" << std::endl
			<< "    drclient_t mClient;" << std::endl
			<< "    char mInitialWorkingDir[PATH_MAX];" << std::endl
			<< "    char mWorkingDir[PATH_MAX];" << std::endl
			<< "    char* mWorkDescription;" << std::endl
			<< "    uint32 mWorkDescriptionSize;" << std::endl
			<< "    char* mBinaryName;" << std::endl
			<< "" << std::endl
			<< "    char** mParameters;" << std::endl
			<< "    uint32 mParameterCurrent;" << std::endl
			<< "    uint32 mParameterCount;" << std::endl
			<< "" << std::endl
			<< "    droperation_t* mImports;" << std::endl
			<< "    uint32 mImportCurrent;" << std::endl
			<< "    uint32 mImportCount;" << std::endl
			<< "" << std::endl
			<< "    droperation_t mExport;" << std::endl
			<< "    droperation_t mAdmin;" << std::endl
			<< "    uint8 mXmlErrorDetected;" << std::endl
			<< "};" << std::endl
			<< "typedef struct worker* worker_t;" << std::endl
			<< "extern int worker_init(worker_t* context, char* data, uint32 dataSize, drclient_t client);" << std::endl
			<< "extern int worker_import_resources(worker_t context);" << std::endl
			<< "extern int worker_export_resouces(worker_t context);" << std::endl
			<< "extern void main_" << impl.getName() << "(int argc,char **argv);" << std::endl
			<< "}" << std::endl;
	out << "Define " << impl.getComponentAbstractName() << "(mode_in int dataSize, mode_in char data[dataSize], mode_out int status[1])" << std::endl
			<< "{" << std::endl
			<< "  int  i,n,status;" << std::endl
			<< "  char **arg;" << std::endl
			<< "  char *dir;" << std::endl
			<< "  worker_t context;" << std::endl
			<< "  drclient_t client=0;" << std::endl
			<< "  if(__myrank==0){" << std::endl
			<< "    status=worker_init(&context,data,dataSize,0);" << std::endl
			<< "    if(status!=0) exit(1);" << std::endl
			<< "    status=worker_import_resources(context);" << std::endl
			<< "    if(status!=0) exit(1);" << std::endl
			<< "    for(n=0;context->mParameters[n];n++);" << std::endl
			<< "  }" << std::endl
			<< "  MPI_Bcast(&n,1,MPI_INT,0,MPI_COMM_WORLD);" << std::endl
			<< "  dir=(char *)malloc(PATH_MAX*sizeof(char));" << std::endl
			<< "  arg=(char **)xmp_malloc_matrix(n+1,PATH_MAX,sizeof(char));" << std::endl
			<< "  if(__myrank==0){" << std::endl
			<< "    for(i=0;i<n;i++){" << std::endl
			<< "      strcpy(arg[i],context->mParameters[i]);" << std::endl
			<< "    }" << std::endl
			<< "    strcpy(arg[n],context->mWorkingDir);" << std::endl
			<< "  }" << std::endl
			<< "  MPI_Bcast(&(arg[0][0]),(n+1)*PATH_MAX,MPI_CHAR,0,MPI_COMM_WORLD);" << std::endl
			<< "  strcpy(dir,arg[n]);" << std::endl
			<< "  chdir(dir);" << std::endl
			<< "  xmp_start_component(n+1,arg);" << std::endl
			<< "  main_" << impl.getName() << "(n+1,arg);" << std::endl
			<< "  MPI_Barrier(MPI_COMM_WORLD);" << std::endl
			<< "  xmp_free_matrix((void ***)&arg,n+1,PATH_MAX,sizeof(char));" << std::endl
			<< "  xmp_finish_component();" << std::endl
			<< "    chdir(context->mInitialWorkingDir);" << std::endl
			<< "  if(__myrank==0){" << std::endl
			<< "    status=worker_export_resources(context);" << std::endl
			<< "    chdir(context->mInitialWorkingDir);" << std::endl

			<< "  }" << std::endl
			<< "}" << std::endl;
	return true;
}

bool XMPBinaryGenerator::createSourceCode(const Yml::Core::ComponentImpl& impl, const Yml::Core::ParameterList& interface, const Yml::Core::string& dest)
{
	// Create the source code
	std::ofstream out(dest.c_str());
	if (!out)
		return false;
	out << "/**********************************************" << std::endl
			<< " * This file has been generated automatically *" << std::endl
			<< " * by the yml_component program               *" << std::endl
			<< " * Please do not edit                         *" << std::endl
			<< " **********************************************/" << std::endl
			<< "// Parameter type include" << std::endl;
	out << "// user provided header" << std::endl
			<< impl.getImplHeader() << std::endl
			<< "// end of user provided header" << std::endl << std::endl;
	// Include all type definition
	out << "#include \"yml_xmp_component_base.hh\"" << std::endl;
	for (size_t i = 0 ; i < interface.count() ; ++i)
	{
		const Yml::Core::Parameter& p = interface.get(i);
		out << "#include \"" <<p.getType() << ".xmptype.h\"" << std::endl;
	}


	out << "// Typedefs for parameters" << std::endl;
	for (size_t i = 0 ; i < interface.count() ; ++i)
	{
		const Yml::Core::Parameter& p = interface.get(i);
		if(p.getMapping().empty()==false)
		{
			int  dim_node; // sizes for xmp-array(s) and dimension of node

			/* Integration of implementation component extension */
			out << "#pragma xmp nodes " << p.getName() << "_nodes(" << impl.getImplNodesTopology() << ")"<< std::endl;

			dim_node=1;
			std::string vNodesTopo = impl.getImplNodesTopology();
			for(std::string::iterator iter = vNodesTopo.begin(); iter != vNodesTopo.end(); iter++) /* To modify */
			{
				if(*iter == ',') dim_node++;
			}

			/* Generating template */

			out << "#pragma xmp template " << p.getName() << "_template(";

			std::vector<int> vSize = p.getSize();
			int k =0;
			if(vSize.size() < dim_node )
			{
				std::vector<int>::iterator iter = vSize.begin();
				for(k=0; k < dim_node; k++)
				{
					if(k != 0) out << ",";
					out << "0:" << *iter-1;
				}
			}
			else
			{
				for(std::vector<int>::iterator iter = vSize.begin(); iter != vSize.end() && k < dim_node; iter++,k++)
				{
					if(iter !=vSize.begin()) out << ",";
					out << "0:" << *iter-1;
				}
			}

			out  << ")" << std::endl;

			/* Generating distribution - data mapping */

			out << "#pragma xmp distribute " << p.getName() << "_template(";

			std::vector<Yml::Core::ParameterMapping> vMap = p.getMapping();
			for(std::vector<Yml::Core::ParameterMapping>::iterator iter = vMap.begin(); iter != vMap.end(); iter++)
			{
				if(iter !=vMap.begin()) out << ",";

				switch(*iter)
				{
				case Yml::Core::PARAM_MAP_BLOCK:
					out << "block";
					break;
				case Yml::Core::PARAM_MAP_CYCLIC:
					out << "cyclic";
					break;
				}
			}

			out << ") onto " << p.getName() << "_nodes" << std::endl;

			/* Generating variable */

			out << "XMP_" << p.getType() << " " << p.getName();

			for(std::vector<int>::iterator iter = vSize.begin(); iter != vSize.end(); iter++)
				out << "[" << *iter << "]" ;
			out << ";" << std::endl;

			/* ----------------------------------- */


			out << "#pragma xmp align " << p.getName() << p.getDataAlign() << " with " << p.getName() << "_template" << p.getTemplateAlign() << std::endl;
			out << std::endl;

		}
		else
		{
			out << "typedef " << p.getType() << " " << p.getName() << "_type;" << std::endl;
		}
		/*
            if (p.isCollection())
                {
                    out << "typedef yml::Collection< yml::" << p.getType() << " > " 
                        << p.getName() << "_type;" << std::endl;
                }
            else
                {
                    out << "typedef yml::" << p.getType() << " " << p.getName() << "_type;" 
                        << std::endl;
                }
		 */
	}
	out << "int main_" << impl.getName() << "(int argc, char **argv)" << std::endl
			<< "{" << std::endl;

	for(size_t i=0;i<interface.count();++i)
	{
		const Yml::Core::Parameter& p = interface.get(i);
		if(p.getMapping().empty())
		{
			out << "  " << p.getName() << "_type " << p.getName() << ";" << std::endl;
		}
	}
	out << std::endl;

	/* Import Parameters */

	for(size_t i=0;i<interface.count();++i){
		const Yml::Core::Parameter& p = interface.get(i);
		if(p.getIOMode()==Yml::Core::PARAM_IO_IN  || p.getIOMode()==Yml::Core::PARAM_IO_INOUT ){
			if(!p.getMapping().empty())
			{

				out << "type_import" << "((void*)" << p.getName() << ",argv[" << i+1 << "]," << p.getType() << "_MPI_Type," << p.getType() <<"_import,\"";

				std::vector<int> vSize = p.getSize();
				for(std::vector<int>::iterator iter = vSize.begin(); iter != vSize.end(); iter++)
				{
					if(iter !=vSize.begin()) out << ",";
					out << *iter ;
				}
				out << ";" << impl.getImplNodesTopology() << ";";

				std::vector<Yml::Core::ParameterMapping> vMap = p.getMapping();
				for(std::vector<Yml::Core::ParameterMapping>::iterator iter = vMap.begin(); iter != vMap.end(); iter++)
				{
					if(iter !=vMap.begin()) out << ",";

					out << *iter;
				}

				out << ";";
				out << p.getDataAlign() << ":" << p.getTemplateAlign() << "\");" << std::endl;


			}
			else
			{
				out << p.getType() << "_import(&" << p.getName()  << ",argv[" << i+1 << "]);" << std::endl;
			}
		}
	}

	out << std::endl;

	// Class body goes here
	out << "{ // ==================== XMP source code : start ====================" << std::endl
			<< impl.getImplSource() << std::endl
			<< "} // ==================== XMP source code :  end  ====================" << std::endl;
	out << std::endl;

	/* Export Parameters */

	for(size_t i=0;i<interface.count();++i)
	{
		const Yml::Core::Parameter& p = interface.get(i);
		if(p.getIOMode()==Yml::Core::PARAM_IO_OUT  || p.getIOMode()==Yml::Core::PARAM_IO_INOUT )
		{
			if(!p.getMapping().empty())
			{

				out << "    type_export((void*)" << p.getName() << ",argv[" << i+1 << "]," << p.getType() << "_MPI_Type," << p.getType() << "_export,\"";

				std::vector<int> vSize = p.getSize();
				for(std::vector<int>::iterator iter = vSize.begin(); iter != vSize.end(); iter++)
				{
					if(iter !=vSize.begin()) out << ",";
					out << *iter ;
				}
				out << ";" << impl.getImplNodesTopology() << ";";

				std::vector<Yml::Core::ParameterMapping> vMap = p.getMapping();
				for(std::vector<Yml::Core::ParameterMapping>::iterator iter = vMap.begin(); iter != vMap.end(); iter++)
				{
					if(iter !=vMap.begin()) out << ",";
					out << *iter;
				}

				out << ";";
				out << p.getDataAlign() << ":" << p.getTemplateAlign() << "\");" << std::endl;

			}
			else
			{
				out << p.getType() << "_export(&" << p.getName() << ",argv[" << i+1 << "]);" << std::endl;
			}
		}
	}


	out << std::endl;

	out << "  return EXIT_SUCCESS; " << std::endl;
	out << "} // end of main component " << std::endl;
	out  << "// User provided footer" << std::endl
			<< "#line 1 \"Footer section\"" << std::endl
			<< impl.getImplFooter() << std::endl
			<< "/***********************************************/" << std::endl;
	out.close();
	return true;
}

bool XMPBinaryGenerator::createRex(const Yml::Core::string& libraries, const Yml::Core::string& dest, const Yml::Core::string& src)
{
	bool abort = false;
	Yml::Core::string xmpcc            = XMP_CC;
	Yml::Core::string xmp_lib          = XMP_LIB;
	Yml::Core::string xmp_std_lib      = mDataDir+"lib/libxmp_std_lib.a";
	Yml::Core::string mpicc            = MPI_CC;
	Yml::Core::string dest_rex_c(dest+".rex.c");
	Yml::Core::string dest_rex_obj(dest+".rex.o");
	Yml::Core::string workerIncludeDir = YML_WORKER_INCLUDE;
	Yml::Core::string yml_worker_lib   = YML_WORKER_LIB;
	Yml::Core::string configIncludeDir = YML_CONFIG_INCLUDE;
	Yml::Core::string omrpc_gen        = OMRPC_GEN_XMP;
	Yml::Core::string omrpcIncludeDir  = OMRPC_INCLUDE;
	Yml::Core::string omrpc_lib_stub   = OMRPC_LIB_STUB;
	Yml::Core::string omrpc_lib_io     = OMRPC_LIB_IO;
	Util::StringList paramList1;
	Util::StringList paramList2;
	Util::StringList paramList3;
	Util::StringList cflagsList;
	Util::StringList ldflagsList;
	Yml::Core::string libInfoDir(mDataDir + "lib" + Util::RuntimeEnvironment::PATH_SEPARATOR);

	paramList1 << src << dest_rex_c;

	Util::Process omrpc;
	omrpc.start(omrpc_gen, paramList1);
	omrpc.wait();
	UTIL_DEBUG("default", "DefaultExecutionCatalog: Compiler completes with exit status " << omrpc.exitStatus());

	paramList2 << "-c" << dest_rex_c << "-I"+workerIncludeDir << "-I"+configIncludeDir << "-I"+omrpcIncludeDir << "-I"+mDataDir+"include" << "-o" << dest_rex_obj;
	omrpc.start(mpicc,paramList2);
	omrpc.wait();

        // ldflagsList << "-static" << "-L" + mDataDir + "lib";
        // Check for includes / libs for all libraries
        if (!libraries.empty())
        {
                std::istringstream libList(libraries);
                while(!libList.eof())
                {
                        Yml::Core::string lib;
                        libList >> lib;
                        UTIL_INFO("default", "DefaultExecutionCatalog: Retrieving information for library '" << lib << "'");
                        std::ifstream libdesc((libInfoDir + lib + ".txt").c_str());
                        if (!libdesc)
                        {
                                UTIL_ERROR("default", "Library '" << lib << "' is not available");
                                abort = true;
                                break;
                        }

                        Yml::Core::string cflags;
                        std::getline(libdesc, cflags);
/*
                        std::istringstream ifl(Util::StringHelper::trim(cflags));
                        while(!ifl.eof())
                        {
                                Yml::Core::string tmp;
                                ifl >> tmp;
                                // Skip the line
                                //if (ifl && ! tmp.empty())  cflagsList << tmp;
                        }
*/
                        Yml::Core::string libs;
                        std::getline(libdesc, libs);
                        std::istringstream ili(Util::StringHelper::trim(libs));
                        while(!ili.eof())
                        {
                                Yml::Core::string tmp;
                                ili >> tmp;
                                if (ili && !tmp.empty()) ldflagsList << tmp;
                        }
                }
        }

	paramList3 << dest_rex_obj << xmp_std_lib << dest+".o" << omrpc_lib_stub << omrpc_lib_io << xmp_lib << yml_worker_lib << "-o" << dest+".rex" << "-lexpat" << "-lm" << ldflagsList;
	omrpc.start(mpicc,paramList3);
	omrpc.wait();

	UTIL_DEBUG("default", "DefaultExecutionCatalog: Compiler completes with exit status " << omrpc.exitStatus());

	return omrpc.exitStatus() == 0;
}


bool XMPBinaryGenerator::createBinary(const Yml::Core::string& libraries, const Yml::Core::string& dest, const Yml::Core::string& src)
{
	// Prepare argument list
	bool abort = false;
	Yml::Core::string compiler = XMP_CC;
	Util::StringList paramList;
	Util::StringList cflagsList;
	Util::StringList ldflagsList;
	Yml::Core::string libInfoDir(mDataDir + "lib" + Util::RuntimeEnvironment::PATH_SEPARATOR);
	cflagsList << "--debug" << "-c" << "-Wall" << "-O2" << "-I" + mDataDir + "include";
	ldflagsList << "-static -lm" << "-L" + mDataDir + "lib";
	// Check for includes / libs for all libraries
	if (!libraries.empty())
	{
		std::istringstream libList(libraries);
		while(!libList.eof())
		{
			Yml::Core::string lib;
			libList >> lib;
			UTIL_INFO("default", "DefaultExecutionCatalog: Retrieving information for library '" << lib << "'");
			std::ifstream libdesc((libInfoDir + lib + ".txt").c_str());
			if (! libdesc)
			{
				UTIL_ERROR("default", "Library '" << lib << "' is not available");
				abort = true;
				break;
			}

			Yml::Core::string cflags;
			std::getline(libdesc, cflags);
			std::istringstream ifl(Util::StringHelper::trim(cflags));
			while(!ifl.eof())
			{
				Yml::Core::string tmp;
				ifl >> tmp;
				if (ifl && ! tmp.empty())
					cflagsList << tmp;
			}

			Yml::Core::string libs;
			std::getline(libdesc, libs);
			std::istringstream ili(Util::StringHelper::trim(libs));
			while(!ili.eof())
			{
				Yml::Core::string tmp;
				ili >> tmp;
				//if (ili && !tmp.empty()) ldflagsList << tmp;
			}
		}
	}

	if (abort)
		return false;

	Yml::Core::string dest_obj=dest+".o";
	Yml::Core::string omrpcIncludeDir=OMRPC_INCLUDE;

	paramList << "-I"+omrpcIncludeDir << cflagsList << "-o" << dest_obj << src << ldflagsList;
	UTIL_DEBUG("default", "DefaultExecutionCatalog: " << compiler << " " << paramList);
	Util::Process cxx;
	cxx.start(compiler, paramList);
	cxx.wait();
	UTIL_DEBUG("default", "DefaultExecutionCatalog: Compiler completes with exit status " << cxx.exitStatus());
	return cxx.exitStatus() == 0;
}
}

