/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/
/**
 * @file
 * @brief 
 * This file contains the supporting library of the component generator for the C++ language. 
 *
 * This file contains the supporting library of the component generator for 
 * the C++ language. Each component generated using CXXGenerator rests on this 
 * header only library to build components. This file allows for a better 
 * separation between the generic code generated by the component and the 
 * common api to develop components. 
 *  
 * 2007-01-30
 * Olivier Delannoy
 *
 *
 */
#ifndef YML_CXX_COMPONENT_BASE_HH
#define YML_CXX_COMPONENT_BASE_HH 1
#include <string>
#include <cstdlib>
#include <cstdio>
#include <fstream>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <algorithm> 
#include <iterator> 
#include <map>
#include <vector>
#include <sys/utsname.h>
#include <sys/time.h>
#include <dirent.h> 
#include <glob.h>
#include <unistd.h> 
#include <stdint.h>
#include <cassert>
#include <cstring>

// Forward declaration 
namespace yml 
{
class Stream;
class FileStream;
class MemoryStream;
class Serializer;
typedef long long CollectionIndex;
typedef std::vector<CollectionIndex> CollectionIndices;
template <typename T> class CollectionResource;
template <typename T> class CollectionNode;
template <typename T> class CollectionNodeIterator;
template <typename T> class Collection;
template <typename T> class CollectionHelper;

template <typename T> bool param_import(T& param, const char* filename);
template <typename T> bool param_export(const T& param, const char* filename);
template <typename T> class IOPolicy_in;
template <typename T> class IOPolicy_out;
template <typename T> class IOPolicy_inout;
template <typename T, typename IOPolicy > class Parameter;
class Component;

////////////////////////////////////////////////////////////
// Serialization helper 
////////////////////////////////////////////////////////////
enum StreamMode
{
    STREAM_READ_ONLY = 1,  
    STREAM_WRITE_ONLY = 2,
    STREAM_READ_WRITE = 3,
};

enum StreamPosMode
{
    STREAM_POS_SET = SEEK_SET, 
    STREAM_POS_CUR = SEEK_CUR, 
    STREAM_POS_END = SEEK_END,
};

////////////////////////////////////////////////////////////
// Abstract base Stream class 
////////////////////////////////////////////////////////////
class Stream
{
protected:
    Stream(StreamMode mode) : d_error(false), d_mode(mode) {}
public:
    
    
    virtual ~Stream() throw() {}
    Stream& write(const char* data, int dataLength)
    {
        if (!d_error)
        {   
            
            
            if (d_mode & STREAM_WRITE_ONLY)
            {
                int written = 0;
                int status = 0;
                while(written < dataLength) 
                {
                    status = writeImpl(data + written, dataLength - written);
                    
                    if (status == -1) 
                    {   
                        d_error = true;
                        break;
                    }
                    else
                    {
                        written += status;
                    }
                }
            }
            else 
            {
                d_error = true;
            }
        }   
        return *this;
    }   
    Stream& read(char* data, int dataLength, int& dataRead)
    {   
        if (!d_error)
        {   
            if (d_mode & STREAM_READ_ONLY)
            {
                int readed = readImpl(data, dataLength);
                //std::cout << "Readed: " << readed << std::endl;
                
                if (readed == -1) 
                {
                    d_error = true;
                    dataRead = 0;
                }
                else 
                {
                    dataRead = readed;
                }
            }
            else
            {
                d_error = true; // Bad stream type 
            }
        }
        return *this;
    }
    Stream& read(char* data, int dataLength)
    {
        int tmp = 0;
        read(data, dataLength, tmp);
        if (tmp != dataLength)
            d_error = true;
        return *this;
    }
    
    Stream& pos(StreamPosMode mode, int offset)
    {
        if (!d_error)
        {
            int status = posImpl(mode, offset);
            if (status == -1)
                d_error = true;
        }
        return *this;
    }
    operator const void*() const { return d_error ? 0 : this; }
    bool operator!() const { return (d_error ? true : false) ;}

    void resetErrorState() { d_error = false;}
protected:
    virtual int writeImpl(const char* data, int dataLength) = 0;
    virtual int readImpl(char* data, int dataLength) = 0;
    virtual int posImpl(StreamPosMode mode, int offset) = 0;
protected:
    bool d_error;
private:
    StreamMode d_mode;

private:
    //Disabled operation 
    Stream(const Stream&);
    Stream& operator=(const Stream&);
};

////////////////////////////////////////////////////////////
// File Stream 
////////////////////////////////////////////////////////////

class FileStream : public Stream
{
public:
    FileStream(const char* filename, StreamMode mode = STREAM_READ_WRITE)
        : Stream(mode), d_fd(0)
    {
        switch(mode)
        {
        case STREAM_READ_ONLY:
            d_fd = ::fopen(filename, "rb");
            break;
        case STREAM_WRITE_ONLY:
            d_fd = ::fopen(filename, "wb");
            break;
        case STREAM_READ_WRITE:
            d_fd = ::fopen(filename, "r+b");
            break;
        }
        if (!d_fd)
            d_error = true;
    }

    ~FileStream() throw()
    {
        fclose(d_fd);
    }
protected:
    int writeImpl(const char* data, int dataLength)
    {
        if (!d_fd)
            return -1;
        return fwrite(data, 1, dataLength, d_fd);
    }
    int readImpl(char* data, int dataLength)
    {
        if (!d_fd)
            return -1;
        return fread(data, 1, dataLength, d_fd);
    }
    int posImpl(StreamPosMode mode, int offset)
    {
        if (!d_fd)
            return -1;
        return fseek(d_fd, mode, offset);    
    }
private:
    FILE* d_fd;
};

////////////////////////////////////////////////////////////
// Memory Stream 
////////////////////////////////////////////////////////////
class MemoryStream : public Stream
{
public:
    explicit MemoryStream(int initialCapacity = 8192)
        : Stream(STREAM_READ_WRITE),
          d_offset(0),
          d_data(new char[initialCapacity]),
          d_size(initialCapacity),
          d_owner(true)
    {

    }
    MemoryStream(char* data, int dataLength)
        : Stream(STREAM_READ_WRITE),
          d_offset(0),
          d_data(data),
          d_size(dataLength),
          d_owner(false)
    {
    }
    ~MemoryStream() throw()
    {
        if(d_owner)
            delete [] d_data;
    }

    const char* getDataPtr() const {return d_data;}
    int getDataSize() const {return d_offset;}

protected:
    int writeImpl(const char* data, int dataLength)
    {
        int avail = d_size - d_offset;
        if (avail == 0)
            grow();
        avail = d_size - d_offset;
        if (avail == 0)
        {
            return 0;
        }
        else if (avail < dataLength)
        {
            memcpy(d_data + d_offset, data, avail);
            d_offset += avail;
            return avail;
        }
        else
        {
            memcpy(d_data + d_offset, data, dataLength);
            d_offset += dataLength;
            return dataLength;
        }
    }
    int readImpl(char* data, int dataLength)
    {
        int avail = d_size - d_offset;
        if (avail == 0)
        {
            return 0;
        }
        else if (avail < dataLength)
        {
            memcpy(data, d_data + d_offset, avail);
            d_offset += avail;
            return avail;
        }
        else
        {
            memcpy(data, d_data + d_offset, dataLength);
            d_offset += dataLength;
            return avail;
        }
    }
    int posImpl(StreamPosMode mode, int offset)
    {
        int newOffset = -1;
        switch(mode)
        {
        case STREAM_POS_SET:
            newOffset = offset;
            break;
        case STREAM_POS_CUR:
            newOffset = d_offset + offset;
            break;
        case STREAM_POS_END:
            newOffset = d_size + offset;
            break;
        };
        if (newOffset >= 0)
        {
            while (d_owner && newOffset > d_size)
                grow();
            d_offset = newOffset;
        }
        return newOffset;
    }

private:
    void grow()
    {
        if (d_owner)
        {
            char* tmp = new char[d_size * 4];
            memcpy(tmp, d_data, d_offset);
            delete [] d_data;
            d_data = tmp;
            d_size*= 4;
        }
    }

private:
    int d_offset;
    char* d_data;
    int d_size;
    bool d_owner;
};

////////////////////////////////////////////////////////////
// Serialization helper class 
////////////////////////////////////////////////////////////
class Serializer
{
public:
    Serializer(Stream& stream) : d_stream(stream) {}

    virtual ~Serializer() throw() {}

    Serializer& readChar(char &value)
    {
        d_stream.read(reinterpret_cast<char*>(&value), sizeof(char));
        return *this;
    }
    Serializer& readUChar(unsigned char& value)
    {
        d_stream.read(reinterpret_cast<char*>(&value), sizeof(unsigned char));
        return *this;
    }
    Serializer& readInt16(int16_t& value)
    {
        d_stream.read(reinterpret_cast<char*>(&value), sizeof(int16_t));
        // add swap 
        return *this;
    }
    Serializer& readUInt16(uint16_t& value)
    {
        d_stream.read(reinterpret_cast<char*>(&value), sizeof(uint16_t));
        // add swap 
        return *this;
    }
    Serializer& readInt32(int32_t& value)
    {
        d_stream.read(reinterpret_cast<char*>(&value), sizeof(int32_t));
        // add swap 
        return *this;
    }
    Serializer& readUInt32(uint32_t& value)
    {
        d_stream.read(reinterpret_cast<char*>(&value), sizeof(uint32_t));
        // add swap
        return *this;
    }
    Serializer& readInt64(int64_t& value)
    {
        d_stream.read(reinterpret_cast<char*>(&value), sizeof(int64_t));
        // add swap
        return *this;
    }
    Serializer& readUInt64(uint64_t& value)
    {
        d_stream.read(reinterpret_cast<char*>(&value), sizeof(uint64_t));
        // add swap
        return *this;
    }
    Serializer& readReal32(float& value)
    {
        d_stream.read(reinterpret_cast<char*>(&value), sizeof(float));
        // add swap
        return *this;
    }
    Serializer& readReal64(double& value)
    {
        d_stream.read(reinterpret_cast<char*>(&value), sizeof(double));
        // add swap
        return *this;
    }
    Serializer& readString(std::string& value)
    {
        uint32_t size;
        readUInt32(size);
        value.resize(size);
        d_stream.read(const_cast<char*>(value.data()), size);
        return *this;
    }
    Serializer& writeChar(char value)
    {
        d_stream.write(reinterpret_cast<char*>(&value), sizeof(char));
        return *this;
    }
    Serializer& writeUChar(unsigned char value)
    {
        d_stream.write(reinterpret_cast<char*>(&value), sizeof(unsigned char));
        return *this;
    }
    Serializer& writeInt16(int16_t value)
    {
        // add swap 
        d_stream.write(reinterpret_cast<char*>(&value), sizeof(int16_t));
        return *this;
    }
    Serializer& writeUInt16(uint16_t value)
    {
        // add swap 
        d_stream.write(reinterpret_cast<char*>(&value), sizeof(uint16_t));
        return *this;
    }
    Serializer& writeInt32(int32_t value)
    {
        // add swap 
        d_stream.write(reinterpret_cast<char*>(&value), sizeof(int32_t));
        return *this;
    }
    Serializer& writeUInt32(uint32_t value)
    {
        // add swap 
        d_stream.write(reinterpret_cast<char*>(&value), sizeof(uint32_t));
        return *this;
    }
    Serializer& writeInt64(int64_t value)
    {
        // add swap 
        d_stream.write(reinterpret_cast<char*>(&value), sizeof(int64_t));
        return *this;
    }
    Serializer& writeUInt64(uint64_t value)
    {
        // add swap 
        d_stream.write(reinterpret_cast<char*>(&value), sizeof(uint64_t));
        return *this;
    }
    Serializer& writeReal32(float value)
    {
        // add swap 
        d_stream.write(reinterpret_cast<char*>(&value), sizeof(float));
        return *this;
    }
    Serializer& writeReal64(double value)
    {
        // add swap 
        d_stream.write(reinterpret_cast<char*>(&value), sizeof(double));
        return *this;
    }
    Serializer& writeString(const std::string& value)
    {
        writeUInt32(value.size());
        d_stream.write(value.data(), value.size());
        return *this;
    }
private:
    Stream& d_stream;
};



////////////////////////////////////////////////////////////
// Component class 
////////////////////////////////////////////////////////////
class Component
{
public:
    static Component& getSingleton() 
    {
        assert(s_instance != 0);
        return *s_instance;
    }   

protected:    
    Component(const char* name, const char* abstract, int argc, char** argv)
        : d_name(name), 
          d_abstractName(abstract), 
          d_argc(argc), 
          d_argv(argv)
    {
        assert(s_instance == 0);
        s_instance = this;
        // Prepare the log file 
        close(1);
        close(2);
        d_log.open("trace.txt", std::ios_base::out | std::ios_base::app);
        dup(1);
        d_output.open("output.txt");
        d_status.open("status.txt");
        if (gettimeofday(&d_startTime, 0) != 0)
        {
            memset(&d_startTime, 0, sizeof(d_startTime));
        }
        if (uname(&d_hostInfo) != 0)
        {
            memset(&d_hostInfo, 0, sizeof(d_hostInfo));
        }
    }
public:
    virtual ~Component()
    {
        d_status.close();
        d_output.close();
        d_log.close();
        assert(s_instance == this);
        s_instance = 0;
    }
    /*!
      \brief 
      Retrieve the name of the implementation component.
      
      Retrieve the name of the implementation component.
    */
    const std::string& getName() const 
    {
        return d_name;
    }
    /*!
      \brief 
      Retrieve the name of the abstract component implemented.
      
      Retrieve the name of the abstract component implemented.
    */
    const std::string& getAbstractName() const
    {
        return d_abstractName;    
    }
    
    void execute()
    {
        initialize();
        try
        {
            executeImpl(d_argc, d_argv);
            success();
        }
        catch(...)
        {
        }
        terminate();
    }
    /*! 
      \brief 
      Log a debugging message 
      
      Log a debugging message 

      \note 
      This function is not thread safe.
    */
    void logDebug(const std::string& debugString)
    {
        
        d_log << "DEBUG " << timestamp() <<": " << debugString << std::endl;
    }
    /*! 
      \brief 
      Log an information level message 
      
      Log an information level message 
      
      \note 
      This function is not thread safe.
    */
    void logInfo(const std::string& infoString)
    {    
        d_log << "INFO " << timestamp() << ": " << infoString << std::endl;
    }
    /*! 
      \brief 
      Log a warning level message 
      
      Log a warning level message 
      \note 
      This function is not thread safe.
    */
    void logWarning(const std::string& warningString)
    {
        d_log << "WARN " << timestamp() << ": " << warningString << std::endl;
    }
    /*! 
      \brief 
      Log an error level message 
      
      Log an error level message 
      \note 
      This function is not thread safe 
    */
    void logError(const std::string& errorString)
    {    
        d_log << "ERROR " << timestamp() << ": " << errorString << std::endl;
    }
    /*! 
      \brief 
      Log an exception 
      
      Log an exception 
      
      \note 
      This function is not thread safe 
    */
    void logException(const std::string& exceptionName)
    {    
        d_log << "EXCEP" << timestamp() << ": " << exceptionName << std::endl;
    }   
    /*! 
      \brief 
      Add a file to the list of resulting files created by the component execution
      
      Add a file to the list of resulting files created by the component execution 
    */
    void addOutputFile(const std::string& filename)
    {
        d_output << filename << std::endl;
    }
    /*! 
      \brief 
      Return a timestamp in UTC time.

      Return a timestamp in UTC time. 

      \note 
      This function is not thread safe.
    */
    static std::string timestamp()
    {
        time_t tmp = time(0);
        char * res = asctime(gmtime(&tmp));
        return std::string(res, strlen(res) - 1);
    }
    std::ostream& out() { return d_log;}    
private:
    /*! 
      \brief 
      This function is used to report successfull component execution.
      
      This function is used to report successfull component execution.
    */
    void success()
    {
        d_status << "success" << std::endl;
    }
protected:
    /*!
      \brief 
      Use this function to abort the execution of a component due to error 
      
      Use this function to abort the execution of a component due to error 
    */
    void error()
    {
        logError("Execution detect an error, component execution aborted");
        d_status << "error" << std::endl;
        throw 1; // Fatal 
    }
    /*!
      \brief 
      Use this function to report an exception to the application.
      
      Use this function to report an exception to the application.
    */
    void exception(const char* name)
    {
        logException(name);
        d_status << "exception" << std::endl
                 << name << std::endl;
        throw 1;
    }
    /*!
      \brief 
      Implement this method in the component generator.
      
      Implement this method in the component generator.
    */
    virtual void executeImpl(int argc, char** argv) = 0;
private:
    void initialize()
    {
        // Print host info 
        d_log << "COMPONENT: " << d_name << "(" << d_abstractName << ")" << std::endl;
        for (int i = 1 ; i < d_argc ; ++i)
            d_log << "COMPONENT: param " << std::setw(4) << i << ": "  << d_argv[i] << std::endl;
        d_log << "COMPONENT: Processor family: " << d_hostInfo.machine << std::endl;
        d_log << "COMPONENT: Operating system: " << d_hostInfo.sysname << std::endl;
        d_log << "COMPONENT: Operating system release: " << d_hostInfo.release << std::endl;
        d_log << "COMPONENT: Operating system version: " << d_hostInfo.version << std::endl;        
        d_log << "COMPONENT: Hostname: " << d_hostInfo.nodename << std::endl;
        d_log << "COMPONENT: start time: " << asctime(gmtime(&d_startTime.tv_sec));
        d_log << "COMPONENT: execution started" << std::endl;
    }
    
    void terminate()
    {
        // Print time info 
        struct timeval endTime;
        if (gettimeofday(&endTime, 0) == 0)
        {
            d_log << "COMPONENT: end time: " << asctime(gmtime(&endTime.tv_sec));
            size_t end = endTime.tv_sec * 1000 + endTime.tv_usec;
            size_t beg = d_startTime.tv_sec * 1000 + d_startTime.tv_usec;
            d_log << "COMPONENT: number of seconds: " << endTime.tv_sec - d_startTime.tv_sec << std::endl;
            d_log << "COMPONENT: number of micro seconds: " << end - beg << std::endl;
            d_log << "COMPONENT: execution finished" << std::endl;
        }
        
    }
    
    std::string d_name;         //!< Store the name of the component being executed 
    std::string d_abstractName; //!< Store the name of the abstract component implemented 
    int d_argc;                 //!< Store the command line information
    char** d_argv;              //!< Store the command line information
    std::string d_hostname;     //!< Store the name of the host
    std::ofstream d_log;        //!< Logger stream
    std::ofstream d_output;     //!< List output file of the component execution 
    std::ofstream d_status;     //!< Store component execution status 
    struct utsname d_hostInfo;  //!< Store host information
    struct timeval d_startTime; //!< Store the starting time of the component execution 
    static Component* s_instance; //! Store singleton instance 
};
Component* Component::s_instance = 0;
///////////////////////////////////////////////////////////
// Collection resource
////////////////////////////////////////////////////////////



template <typename T>
class CollectionResource 
{
public:
    explicit CollectionResource(const std::string& filename)
        : d_name(filename)
    {
        
        Component::getSingleton().logDebug("New Collection resource added: " + filename);
        const size_t max = d_name.size();
        d_yvetteName.reserve(max);
        bool findAt = false;
        bool first = true;
        
        for(size_t i = 0; i < max ; ++i)
        {
            if (findAt && d_name[i] == '@')
            {
                if (first)
                {
                    d_yvetteName += '[';
                    findAt = false;
                    first = false;
                }
                else 
                {
                    d_yvetteName += "][";
                    findAt = false;
                }
            }
            if (d_name[i] != '@')
                d_yvetteName += d_name[i];
            else 
                findAt = true;
        }
        d_yvetteName += ']';    
    }
    const std::string& getYvetteName() const
    {
        return d_yvetteName;
    }
    const std::string& getFileName() const
    {
        return d_name;
    }
    bool hasBeenGet() const
    {
        return d_hasBeenImported;
    }
    bool hasBeenSet() const
    {
        return d_hasBeenExported;
    }
    bool get(T& value)
    {
        if ( param_import(value, d_name.c_str()) ) 
        {
            d_hasBeenImported = true;
            return true;
        }
        return false;
        
    }
    bool set(const T& value)
    {
        if( param_export(value, d_name.c_str()) )
        {
            d_hasBeenExported = true;
            return true;
        }
        return false;       
    }
private:
    std::string d_name;
    std::string d_yvetteName;
    bool d_hasBeenImported;
    bool d_hasBeenExported;
};

////////////////////////////////////////////////////////////
// Collection Node 
////////////////////////////////////////////////////////////
template <typename T>
class CollectionNode
{
public:
    ~CollectionNode()
    {
        typename std::map<CollectionIndex, CollectionNode<T>*>::iterator iter = d_childs.begin();
        const typename std::map<CollectionIndex, CollectionNode<T>*>::iterator end = d_childs.end();
        for (; iter != end ;  ++iter)
            delete iter->second;
    }
    /*!
      \Brief 
      Create a new child node indexed \em index.
      
      Create a new child node indexed \em index.
      This call does nothing if the node already exists.
    */  
    CollectionNode* newChild(CollectionIndex index)
    {
        CollectionNode* res = getChild(index);
        if (! res)
        {
            // Add the child node (check for child creation authorized 
            if (! d_lock) {
                res = new CollectionNode(d_collection, this, index);
                d_childs[index] = res;
            }
        }
        // Nothing to do if the child node already exists 
        return res;
    }    
    /*!
      \brief 
      Create a new resource associated to this node 
      
      Create a new resource associated to this node. If the resource
      already exists, nothing happens. If this operation is executed
      on the root node of a collection nothing happens too.
    */
    CollectionResource<T>* newResource()
    {
        if (! d_resource && d_parent && !d_lock)
        {
            // Create a resource 
            d_resource = d_collection->newResource(this);
        }
        return d_resource;
        
    }
    /*!
      \brief 
      Destroy an existing resource attached to this node.
      
      Destroy an existing resource attached to this node.
    */
    void deleteResource()
    {
        if (d_resource)
        {
            d_collection->deleteResource(this);
            d_resource = 0;
        }
    }
    /*!
      \brief 
      Return the CollectionResource object associated to that node if any.

      Return the CollectionResource object associated to that node if any.
      The parent node is never associated to a resource.
    */
    CollectionResource<T>* getResource()
    {
        return d_resource;
    }
    /*! 
      \brief
      Get the parent of the current node.
      
      Get the parent of the current node. 
      Only the root node of a collection does not have a parent.
    */
     
    CollectionNode* getParent()
    {
        return d_parent;
    }
    /*!
      \brief 
      Get the child node by \em index 
      
      Get the child node by \em index 
    */
    CollectionNode* getChild(CollectionIndex index)
    {
        typename std::map<CollectionIndex, CollectionNode* >::iterator iter = d_childs.find(index);
        
        if (iter != d_childs.end())
            return iter->second;
        return 0;
    }
    /*!
      \brief 
      Get an iterator to access the child node of the current node.

      Get an iterator to access the child node of the current node.
    */
    CollectionNodeIterator<T> getChildNodeIterator()
    {
        return CollectionNodeIterator<T>(d_childs.begin(), d_childs.end());
    }
    /*!
      \brief 
      Get the index of the node.

      Get the index of the node.
    */
    CollectionIndex getIndex() const
    {
        return d_index;
    }
    /*!
      \brief 
      True if this node is the root node of the collection.
      
      True if this node is the root node of the collection.
      
      if this function return true then the index has no meaning.
    */
    bool isRoot() const
    {
        return d_parent == 0;
    }
    /*!
      \brief say wether a node is lock within the partial collection 
    */
    bool isLook() const 
    {
        return d_lock;
    }
    /*! 
      \brief 
      Build the indices list of all registered child so far. 
      
      Build the indices list of all registered child so far. This
      provide an easy indirect way to iterate over the child of a node
      without using iterators. Using iterator is a lot cheaper. 
    */
    void getIndices(CollectionIndices& indices) const 
    {
        indices.clear();
        indices.reserve(d_childs.size());
        typename std::map<CollectionIndex, CollectionNode<T>* >::const_iterator iter = d_childs.begin();
        const typename std::map<CollectionIndex, CollectionNode<T>* >::const_iterator end = d_childs.end();
        for (; iter != end ; ++iter)
            std::back_inserter(indices) = iter->first;
        
    }
private:
    CollectionNode<T>(Collection<T>* collection, CollectionNode<T>* parent = 0, CollectionIndex index = 0, CollectionResource<T>* resource = 0, bool lock = false) 
        : d_collection(collection), 
          d_parent(parent), 
          d_index(index),
          d_resource(resource), 
          d_lock(lock)
        
    {
    }
    
    Collection<T>* d_collection;
    CollectionNode<T>* d_parent;
    CollectionIndex d_index;
    CollectionResource<T>* d_resource;
    std::map<CollectionIndex, CollectionNode<T>* > d_childs;
    bool d_lock; 
    friend class Collection<T>;
};

////////////////////////////////////////////////////////////
// Collection Node Iterator 
////////////////////////////////////////////////////////////
template <typename T>
class CollectionNodeIterator
{

public:
    typedef typename std::map<CollectionIndex, CollectionNode<T>*>::iterator iterator;
    
    CollectionIndex getIndex() const
    {
        return d_iter->first;
    }
    
    CollectionNode<T>* getNode() const
    {
        return d_iter->second;
    }
    bool hasNext() const
    {
        iterator next = d_iter;
        ++next;
        return next != d_end;
    }
    CollectionNodeIterator<T>& operator++() {
        ++d_iter;
        return *this;
    }

    void next()
    {
        ++d_iter;
    }
    bool operator!() {
        return d_iter == d_end;
    }
    operator void*() {
        if (d_iter == d_end)
            return 0;
        return this;
    }
private:
    CollectionNodeIterator(const iterator& begin, const iterator& end)
        : d_iter(begin), d_end(end)
    {
    }
    iterator d_iter;
    const iterator& d_end;
    friend class CollectionNode<T>;
};    
    
////////////////////////////////////////////////////////////
// Collection
////////////////////////////////////////////////////////////

template <typename T>
class Collection
{
public:
    explicit Collection(const char* basename)
        : d_baseInitial(basename), d_root(new CollectionNode<T>(this, 0, 0))
    {   
        //std::cout << "Initial: " << d_baseInitial << std::endl; 
        std::string::size_type pos = d_baseInitial.find_first_of('@');
        //std::cout << "Index of @@: " << pos << "/" << std::string::npos <<  std::endl; 
        if (pos != std::string::npos)
            d_base = d_baseInitial.substr(0, pos);
        else
            d_base = d_baseInitial;

        //std::cout << "[" << d_baseInitial << "][" << d_base << "]" << std::endl;
    }
    void printResourceMap(std::ostream& out)
    {
        out << d_base << std::endl;
        printResourceMap(out, d_base, d_root);
        out << d_base << std::endl;
    }
    void printResourceMap(std::ostream& out, const std::string& prefix, CollectionNode<T>* node)
    {
        std::ostringstream pref;
        pref << prefix << '/' << node->getIndex();
        out << pref.str() << std::endl;
        /*
        CollectionNodeIterator<T> iter = node->getChildNodeIterator();
        while(iter) {
            printResourceMap(out, pref.str(), iter.getNode());
            iter.next();
        }
        */
        for (typename std::map<CollectionIndex, CollectionNode<T>*>::const_iterator iter = node->d_childs.begin() ; iter != node->d_childs.end() ; ++iter)
            printResourceMap(out, pref.str(), iter->second);
        out << pref.str() << std::endl;
        
    }
    bool importResourceMap()
    {
        // Register locked node 
        //std::cout << "Building resource map" << std::endl; 
        if (d_base != d_baseInitial) { // Partial collection unable locks 
            //std::cout << "Partial collection" << std::endl;
            std::string nodeIndex = d_baseInitial.substr(d_base.size() + 2);
            std::replace_if(nodeIndex.begin(), nodeIndex.end(), std::bind2nd(std::equal_to<char>(), '@') ,' ');
            std::istringstream in(nodeIndex);
            std::vector<int> indexStack;
            indexStack.assign(std::istream_iterator<int>(in), std::istream_iterator<int>());
            const std::vector<int>::const_iterator end = indexStack.end();
            std::vector<int>::const_iterator iter = indexStack.begin();
            CollectionNode<T>* node = d_root;
            for (; iter != end ; ++iter) {
                node = node->newChild(*iter); 
            }
            // Lock all nodes except the last one 
            iter = indexStack.begin();
            node = d_root;
            for (; iter != end - 1; ++iter) {
                node->d_lock = true;
                node = node->getChild(*iter); 
            }
        }
        //std::cout << "Back to standard support" << std::endl; 

        // Parse the current directory and look for all file named 
        // with something like 
        // basename@@....
        std::string globpattern(d_base + "@@*");
        glob_t files;
        if (glob(globpattern.c_str(), 0, NULL, &files) == -1)
        {
            return false;
        }
        const size_t max=files.gl_pathc;
        std::string collectionName ;
        std::vector<int> indexStack;
        CollectionNode<T>* node;
        for(size_t i = 0 ; i < max ; ++i)
        {
            // For each entry split out the string 
            std::string current(files.gl_pathv[i]);
            std::replace_if(current.begin(), current.end(), std::bind2nd(std::equal_to<char>(), '@') ,' ');
            std::istringstream in(current);
            in >> collectionName;
            indexStack.assign(std::istream_iterator<int>(in), std::istream_iterator<int>());
            const std::vector<int>::const_iterator end = indexStack.end();
            std::vector<int>::const_iterator iter = indexStack.begin();
            node = d_root;
            for (; iter != end ; ++iter)
            {
                node = node->newChild(*iter);
            }
            node->newResource();
        }
        globfree(&files);
        

        return true;
    }
    bool exportResourceMap() const
    {
        const typename std::vector<CollectionResource<T>* >::const_iterator end = d_resources.end();
        typename std::vector<CollectionResource<T>* >::const_iterator iter = d_resources.begin();   
        for ( ;iter != end  ;++iter )
        {
            Component::getSingleton().addOutputFile((*iter)->getFileName());
        }
        return true;
        
    }
    
    CollectionNode<T>* getRoot() const
    {
        return d_root;
    }
    CollectionResource<T>* newResource(CollectionNode<T>* node)
    {
        assert(node->d_collection == this);
        if (node->d_resource || node == d_root)
        {
            return node->d_resource;
        }
        else 
        {
            std::ostringstream filename;
            std::vector<CollectionIndex> stack;
            CollectionNode<T>* tmp = node->getParent();
            stack.push_back(node->getIndex());
            filename << d_base;
            while(! tmp->isRoot())
            {
                stack.push_back(tmp->getIndex());
                tmp = tmp->getParent();
            }
            typename std::vector<CollectionIndex >::const_reverse_iterator iter = stack.rbegin();
            const typename std::vector<CollectionIndex >::const_reverse_iterator end = stack.rend();
            for(; iter != end ; ++iter)
            {
                filename << "@@" << *iter;
            }
            CollectionResource<T>* result = new CollectionResource<T>(filename.str());
            d_resources.push_back(result);
            return result;
        }
    }
    
    void deleteResource(const CollectionNode<T>* node)
    {
        assert(node->d_collection == this);
        if (node->getResource())
        {
            typename std::vector<CollectionResource<T>* >::iterator iter = std::find( *node.d_resource, d_resources.begin(), d_resources.end());
            if (iter != d_resources.end())
            {
                delete *iter;
                d_resources.remove(iter);
            }
        }
    }
    size_t getResourceCount() { return d_resources.size();}
protected:
private:
    std::string d_base;         //!< Store the prefix of the collection files
    std::string d_baseInitial;     //! Store the initial base name given at the collection creation time (usefull for partial collection constraints 
    CollectionNode<T>* d_root;     //!< Store the root node of the collection
    std::vector<CollectionResource<T>* > d_resources; //!< Store low level resources without any indexing facilities nor ordering
    
};
/*!  
  \brief 
  CollectionHelper is an added layer on top of the low level
  API to collections.

  CollectionHelper is an added layer on top^of the low level API to
  collections. It provides the user way with an easy to use interface
  to navigate within collections.
*/
template <typename T>
class CollectionHelper
{
public:
    typedef CollectionHelper<T>   self_type;
    typedef T                     value_type;
    typedef CollectionIndices     indices_type;
    typedef Collection<T>         collection_type;
    typedef CollectionResource<T> resource_type;
    typedef CollectionNode<T>     node_type;
    typedef CollectionIndex       index_type;

    explicit CollectionHelper(collection_type& collection) 
        : d_parent(0), 
          d_collection(collection), 
          d_node(collection.getRoot()), 
          d_index(0)
    {
    }
    
    // Radom access operator 
    self_type operator[](index_type childIndex) 
    {
	if (d_node)
	    return CollectionHelper(*this, d_node->getChild(childIndex), childIndex);
	else 
	    return CollectionHelper(*this, 0, childIndex);
    }
    // Checker 
    bool nodeExists() const { return d_node != 0; }
    bool resourceExists() const { return d_node != 0 && d_node->getResource() != 0; }
    bool isRoot() const { return d_node != 0 && d_node->isRoot() ; }

    // Value retrieval 
    bool get(value_type& value) { return d_node != 0 && d_node->getResource() != 0 && d_node->getResource()->get(value) ; }
    bool set(const value_type& value) { createResource(); return d_node->getResource()->set(value) ; }
    
    
    // convertors 
    resource_type& resource() { createResource(); return *(d_node->getResource());}
    node_type& node() { createNode() ; return *d_node; }
    collection_type& collection() { return d_collection;}
    
    void indices(indices_type& indices)
    {
        indices.clear();
        if (d_node)
            d_node->getIndices(indices);
    }
    
    // Creators 
    void createNode() 
    {
        if (d_node == 0) {
            if (d_parent->d_node == 0)
                d_parent->createNode();
            d_node = d_parent->d_node->newChild(d_index);
        }
    }
    
    void createResource()
    {
        if (! d_node)
            createNode();
        d_node->newResource();
    }

    CollectionHelper(const self_type& obj)
        : d_parent(obj.d_parent), 
          d_collection(obj.d_collection), 
          d_node(obj.d_node), 
          d_index(obj.d_index)
    {
    }

private:
    CollectionHelper(self_type& parent, node_type* node, index_type index)
        : d_parent(&parent), 
          d_collection(parent.d_collection), 
          d_node(node), 
          d_index(index)
    {
    }

    self_type* d_parent; 
    collection_type& d_collection;
    node_type* d_node;
    index_type d_index;
};



////////////////////////////////////////////////////////////
// Parameter  import and export generic (double, int, so on)
////////////////////////////////////////////////////////////
template <typename T> 
bool param_import(T& param, const char* filename)
{
    assert(0 && "Oups: No specialization available for param_import");
    return false;
}


template <typename T> 
bool param_export(const T& param, const char* filename)
{
    assert(0 && "Oups: No specialization available for param_export");
    return false;
}



////////////////////////////////////////////////////////////
// Import and export for Collections 
////////////////////////////////////////////////////////////
template <typename T>
bool param_import(Collection<T>& param, const char* filename)
{
    return param.importResourceMap();
}

template <typename T>
bool param_export(const Collection<T>& param, const char* filename)
{
    return param.exportResourceMap();
}


////////////////////////////////////////////////////////////
// Parameter information 
////////////////////////////////////////////////////////////
template <class T, class IOPolicy >
class Parameter
{
public:
    Parameter(T& param)
        : mParam(param), 
          mExists(false),
          mExport(true)
    {
    }   
    inline bool pimport(const char* filename)
    {   
        bool result = true;
        // Check for existancy first 
        mExists = mPolicy.pexists(filename);
        if (mExists)
            result = mPolicy.pimport(mParam, filename);
        if (result)
            Component::getSingleton().logDebug(std::string("Import ") + filename + " successfull");
        else 
            Component::getSingleton().logDebug(std::string("Import ") + filename + " failed");
        return result;
    }   
    inline bool pexport(const char* filename)
    {   
        bool result = true;
        if (mExport)
            result =  mPolicy.pexport(mParam, filename);
        if (result)
            Component::getSingleton().logDebug(std::string("Export ") + filename + " successfull");
        else 
            Component::getSingleton().logDebug(std::string("Export ") + filename + " failed");
        return result;
    }   
    
    // Return the parameter value 
    T& value() 
    {
        return mParam;    
    }
    // Return true if the data file exists 
    // It's not an error to load a non existing parameter 
    bool exists() 
    {
        return mExists;
    }
    // By default export state is set to true 
    // This function allow you to disable export of a parameter at runtime 
    void setExportState(bool doExport = true)
    {
        mExport = doExport;
    }
    
private:
    T& mParam;
    bool mExists;
    bool mExport; 
    IOPolicy mPolicy;
    Parameter(Parameter<T, IOPolicy >&);
    Parameter<T, IOPolicy >& operator=(Parameter<T, IOPolicy >&);
};


////////////////////////////////////////////////////////////
// IO Policies (in, out, inout)
////////////////////////////////////////////////////////////
template <typename T>
class IOPolicy_in
{
public:
    bool pexists(const char* filename)
    {
        FILE* tmp = fopen(filename, "rb");
        if (tmp)
        {
            fclose(tmp);
            return true;
        }
        return false;
    }
    
    bool pimport(T& param, const char* filename)
    {
        return param_import(param, filename);
    }
    bool pexport(const T& param, const char* filename)
    {
        return true;
    }
};

template <typename T>
class IOPolicy_out
{
public:
    bool pexists(const char* filename)
    {
        return false;
    }
    bool pimport(T& param, const char* filename)
    {
        return true;
    }
    bool pexport(const T& param, const char* filename)
    {
        Component::getSingleton().addOutputFile(filename);
        return param_export(param, filename);
    }
};

template <typename T >
class IOPolicy_inout
{
public:
    bool pexists(const char* filename)
    {
                FILE* tmp = fopen(filename, "rb");
        if (tmp)
        {
            fclose(tmp);
            return true;
        }
        return false;
    }
    bool pimport(T& param, const char* filename)
    {
            return param_import(param, filename);
    }
    bool pexport(const T& param, const char* filename)
    {
        Component::getSingleton().addOutputFile(filename);
        return param_export(param, filename);
    }
};
////////////////////////////////////////////////////////////
// Collection specific output policy (out, inout)
////////////////////////////////////////////////////////////

template <typename T>
class IOPolicy_in<Collection<T> > 
{
public:
    bool pexists(const char* filename)
    {
        // Should add a better checking here 
        return true;
    }
    bool pimport(Collection<T>& param, const char* filename)
    {
        return param_import(param, filename);
    }
    bool pexport(const Collection<T>& param, const char* filename)
    {
        return true;
    }
};

    

template <typename T>
class IOPolicy_out<Collection<T> >
{
public:
    bool pexists(const char* filename)
    {
        return false;   
    }
    bool pimport(Collection<T>& param, const char* filename)
    {
        return true;
    }
    bool pexport(const Collection<T>& param, const char* filename)
    {
        return param_export(param, filename);
    }
};


template <typename T>
class IOPolicy_inout<Collection<T> >
{
public:
    bool pexists(const char* filename)
    {
        // Should add a better checking here 
        return true;
    }
    bool pimport(Collection<T>& param, const char* filename)
    {
        return param_import(param, filename);
    }
    bool pexport(const Collection<T>& param, const char* filename)
    {
        return param_export(param, filename);
    }
};
}
#define COMPONENT_CALL(ClassName)               \
    int main(int argc, char** argv)             \
    {                                           \
        ClassName cmp(argc, argv);              \
        cmp.execute();                          \
        return EXIT_SUCCESS;                    \
    }   

#endif
