/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class OrderingFunction function definition 
 *
 * 2005-08-03
 * Olivier Delannoy
 */
#include "OrderingFunction.hh"
#ifndef UTIL_COMPAT_HAVE_INLINE
#define inline
#include "OrderingFunction.icc"
#undef inline
#endif 



Yml::Core::YvetteObject FunctionOperatorLE::executeImpl(void)
{
    if (mParameters.size() != 2)
        return Yml::Core::YvetteObject::INVALID_OBJECT;
    else if (mParameters[0].getType() == Yml::Core::YvetteObject::INTEGER &&
        mParameters[0].getType() == Yml::Core::YvetteObject::INTEGER)
        return Yml::Core::YvetteObject( (mParameters[0].integerValue() <= mParameters[1].integerValue()) ? 1 : 0);
    else if (mParameters[0].getType() != Yml::Core::YvetteObject::STRING &&
        mParameters[0].getType() != Yml::Core::YvetteObject::STRING)
        return Yml::Core::YvetteObject( (mParameters[0].realValue() <= mParameters[1].realValue()) ? 1 : 0);
    else if (mParameters[0].getType() == Yml::Core::YvetteObject::STRING &&
        mParameters[0].getType() == Yml::Core::YvetteObject::STRING)
        return Yml::Core::YvetteObject((mParameters[0].stringValue().compare(mParameters[1].stringValue()) <= 0) ? 1 : 0);
    return Yml::Core::YvetteObject::INVALID_OBJECT;
}

Yml::Core::YvetteObject FunctionOperatorGE::executeImpl(void)
{
    if (mParameters.size() != 2)
        return Yml::Core::YvetteObject::INVALID_OBJECT;
    else if (mParameters[0].getType() == Yml::Core::YvetteObject::INTEGER &&
        mParameters[0].getType() == Yml::Core::YvetteObject::INTEGER)
        return Yml::Core::YvetteObject( (mParameters[0].integerValue() >= mParameters[1].integerValue()) ? 1 : 0);
    else if (mParameters[0].getType() != Yml::Core::YvetteObject::STRING &&
        mParameters[0].getType() != Yml::Core::YvetteObject::STRING)
        return Yml::Core::YvetteObject( (mParameters[0].realValue() >= mParameters[1].realValue()) ? 1 : 0);
    else if (mParameters[0].getType() == Yml::Core::YvetteObject::STRING &&
        mParameters[0].getType() == Yml::Core::YvetteObject::STRING)
        return Yml::Core::YvetteObject((mParameters[0].stringValue().compare(mParameters[1].stringValue()) >= 0) ? 1 : 0);
    return Yml::Core::YvetteObject::INVALID_OBJECT;
}

Yml::Core::YvetteObject FunctionOperatorLT::executeImpl(void)
{
    if (mParameters.size() != 2)
        return Yml::Core::YvetteObject::INVALID_OBJECT;
    else if (mParameters[0].getType() == Yml::Core::YvetteObject::INTEGER &&
        mParameters[0].getType() == Yml::Core::YvetteObject::INTEGER)
        return Yml::Core::YvetteObject( (mParameters[0].integerValue() < mParameters[1].integerValue()) ? 1 : 0);
    else if (mParameters[0].getType() != Yml::Core::YvetteObject::STRING &&
        mParameters[0].getType() != Yml::Core::YvetteObject::STRING)
        return Yml::Core::YvetteObject( (mParameters[0].realValue() < mParameters[1].realValue()) ? 1 : 0);
    else if (mParameters[0].getType() == Yml::Core::YvetteObject::STRING &&
        mParameters[0].getType() == Yml::Core::YvetteObject::STRING)
        return Yml::Core::YvetteObject((mParameters[0].stringValue().compare(mParameters[1].stringValue()) == -1) ? 1 : 0);
    return Yml::Core::YvetteObject::INVALID_OBJECT;
}

Yml::Core::YvetteObject FunctionOperatorGT::executeImpl(void)
{
    if (mParameters.size() != 2)
        return Yml::Core::YvetteObject::INVALID_OBJECT;
    else if (mParameters[0].getType() == Yml::Core::YvetteObject::INTEGER &&
        mParameters[0].getType() == Yml::Core::YvetteObject::INTEGER)
        return Yml::Core::YvetteObject( (mParameters[0].integerValue() > mParameters[1].integerValue()) ? 1 : 0);
    else if (mParameters[0].getType() != Yml::Core::YvetteObject::STRING &&
        mParameters[0].getType() != Yml::Core::YvetteObject::STRING)
        return Yml::Core::YvetteObject( (mParameters[0].realValue() > mParameters[1].realValue()) ? 1 : 0);
    
    else if (mParameters[0].getType() == Yml::Core::YvetteObject::STRING &&
        mParameters[0].getType() == Yml::Core::YvetteObject::STRING)
        return Yml::Core::YvetteObject((mParameters[0].stringValue().compare(mParameters[1].stringValue()) == 1) ? 1 : 0);   
    return Yml::Core::YvetteObject::INVALID_OBJECT;
}



