/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Plugin C export function 
 *
 * 2005-06-15
 * Olivier Delannoy
 */
#include <Yml/Core/YvetteFunctionPackage.hh>
#include "CastFunction.hh"
#include "ArithmeticFunction.hh"
#include "BooleanFunction.hh"
#include "OrderingFunction.hh" 
#include "EqualityFunction.hh"
/**
 * Create a new instance of a Process Backend
 */
extern "C" Yml::Core::YvetteFunctionPackage* create(void)
{
    Yml::Core::YvetteFunctionPackage* package = new Yml::Core::YvetteFunctionPackage;
    // Register all function
    
    // Cast Function 
    package->add("strval", new FunctionStr);
    package->add("intval", new FunctionInt);
    package->add("boolval", new FunctionInt);
    package->add("realval", new FunctionReal);  
    
    // Arithmetic operator 
    package->add("operator_add", new FunctionOperatorAdd);
    package->add("operator_sub", new FunctionOperatorSub);
    package->add("operator_mul", new FunctionOperatorMul);
    package->add("operator_div", new FunctionOperatorDiv);
    package->add("operator_mod", new FunctionOperatorMod);
    
    // Boolean Operator 
    package->add("operator_and", new FunctionOperatorAnd);
    package->add("operator_or", new FunctionOperatorOr);
    package->add("operator_xor", new FunctionOperatorXor);
    package->add("operator_not", new FunctionOperatorNot);

    // Ordering Operator 
    package->add("operator_le", new FunctionOperatorLE);
    package->add("operator_ge", new FunctionOperatorGE);
    package->add("operator_lt", new FunctionOperatorLT);
    package->add("operator_gt", new FunctionOperatorGT);
    
    // Equality Operator 
    package->add("operator_eq",  new FunctionOperatorEq);
    package->add("operator_neq", new FunctionOperatorNeq);
    
    return package;
}

