/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Abstract Syntax Tree  
 *  
 * 2005-06-18
 * Olivier Delannoy
 */
#ifndef YML_CORE_AST_HH
#define YML_CORE_AST_HH
#include "core.hh"
#include <iostream>
namespace Yml {
    namespace Core {
        /**
         * @brief Abstract Syntax Tree Node 
         *
         * It's a generalized interface for Abstract Syntax Tree
         * creation. The AST provide to link, one to the first of it's
         * child and one to the next sibling node. This way any tree
         * can follow this representation. 
         */
        class AST 
        {
        public:
            /** 
             * Default Constructor
             */
            AST(void);
            /**
             * Destructor
             */
            virtual ~AST(void);
            /**
             * Reset all information in this node
             */
            virtual void reset(void);
            /**
             * Retrieve node type 
             */
            uint32 getType(void) const;
            /**
             * Set node type 
             */
            void setType(uint32 type);
            /**
             * Get the line number 
             */
            uint32 getLine(void) const;
            /**
             * Set the line number 
             */
            void setLine(uint32 line);
            /**
             * Retrieve node text
             */
            const string& getText(void) const;
            /**
             * Set node text
             */
            void setText(const string& text);
            /**
             * Retrieve first child node
             */ 
            AST* getFirstChild(void);
            /**
             * Set first child node 
             */
            void setFirstChild(AST* child);
            /**
             * Retrieve next node 
             */
            AST* getNextSibling(void);
            /**
             * Set next node 
             */
            void setNextSibling(AST *nextSibling);
            /**
             * Retrieve parent node 
             */
            AST* getParent(void);
            /**
             * Set parent node 
             */
            void setParent(AST *parent);  
            /**
             * Print all the tree 
             */
            void printTree(std::ostream& out, uint32 level = 0);
        protected:

        private:
            /**
             * Store the text associated to this node 
             */
            string mText;
            /**
             * Store the type of this node 
             */
            uint32 mType;
            /**
             * Store the line number 
             */
            uint32 mLine;
            /**
             * Store a pointer to the parent node 
             */
            AST* mParent;
            /**
             * Store a pointer to the first child node
             */
            AST* mChild;
            /**
             * Store a pointer to the next sibling node 
             */
            AST* mNext;
        };
    }  
}
std::ostream& operator<<(std::ostream& out, Yml::Core::AST& obj);
# ifdef HAVE_INLINE
#  include "AST.icc"
# endif 
#endif 

