/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class Account declaration
 *  
 * 2005-06-16
 * Olivier Delannoy
 */
#ifndef YML_CORE_ACCOUNT_HH
#define YML_CORE_ACCOUNT_HH 1
#include "core.hh"
namespace Yml {
    namespace Core {
        /**
         * @brief This class store all information regarding a user.
         *
         * users are described with this information : 
         * - a login  
         * - a password  
         * - a mail address
         * - the fullname of the user   
         * - the name of organisation  of the user 
         * - a set of rights. 
         * The Account manager is in charge of storing information about users. 
         * 
         * @todo The futur version will also integrates some
         * statistical information. 
         */
        class Account
        {
        public:
            /**
             * Default Contstructor
             */
            Account(void);
            /**
             * Destructor
             */
            ~Account(void);
            /** Get the login */
            const string& getLogin(void) const;
            /** Set the login */
            void setLogin(const string& login);
            /** Get the password */
            const string& getPassword(void) const;
            /** Set the password */
            void setPassword(const string& password);
            /** Get the mail */
            const string& getMail(void) const;
            /** Set the mail */
            void setMail(const string& mail);
            /** Get the name of the user */
            const string& getFullName(void) const;
            /** Set the name of the user */
            void setFullName(const string& name);
            /** Set the organization of the user */
            const string& getOrganization(void) const;
            /** Set organisation name */
            void setOrganization(const string& org);
            /** Set rights of the user */
            void setRights(uint32 rights);
            /** Check if a user can do an action */
            bool isAllowed(uint32 action) const;
            /** Retrieve all rights */
            uint32 getRights(void) const;
            
        protected:
            
        private:
            /** Store login */
            string mLogin;
            /** Store password */
            string mPassword;
            /** Store mail */
            string mMail;
            /** Store name */
            string mName;
            /** Store organisation */
            string mOrg;
            /** Store rights  */
            uint32 mRights;
        };
    }
}
#ifdef HAVE_INLINE
#include "Account.icc"
#endif
#endif

