/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class AccountManager declaration
 *  
 * 2005-06-17
 * Olivier Delannoy
 */
#ifndef YML_CORE_ACCOUNTMANAGER_HH
#define YML_CORE_ACCOUNTMANAGER_HH 1
#include "core.hh"
namespace Yml {
    namespace Core {
        class Account;
        /**
         * @ingroup Plugin_Interface 
         * @brief manage users. 
         *
         * The AccountManager class main role consists in handling
         * request concerning the user accounts. All user account is
         * represented as instance of the class User. 
         * 
         * The most important information concerning users is it's
         * rights that describes what a user can do and what he can
         * not. 
         */
        class AccountManager
        {
        public:
            /**
             * Destructor
             */
            virtual ~AccountManager(void);
            
            /** 
             * Initialise the account manager. The format of initData
             * is up two the implementation. 
             * 
             * @param initData the initialization data.
             *
             * @return If @em false then the AccountManager can not be use at all. @em True on success
             */
            virtual bool init(const string& initData) = 0;
            
            /** 
             * Load data associated to a user 
             * 
             * @param login the login 
             * @param password the password 
             * @return a pointer to a User object 0 otherwise. 
             */
            virtual Account * load(const string& login, const string& password) = 0;
            
            /**
             * Add a new user to the user databases 
             * 
             * @param account A pointer to an Account object 
             * 
             * @return true if the operation is sucessfull false otherwise 
             */
            virtual bool insert(Account *account) = 0;
            
            /**
             * Delete a user 
             * 
             * @param login the login name of the user 
             * 
             * @return true if the operation is successfull false otherwise 
             */
            virtual bool remove(const string& login) = 0;
            
            /**
             * Update user information to the database file 
             * 
             * @param account a pointer to an Account object 
             * 
             * @return true on success false otherwise 
             */
            virtual bool update(Account *account) = 0;
        protected:    
        private:
        };
    }
}
#ifdef HAVE_INLINE
#include "AccountManager.icc"
#endif
#endif

