/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class SchedulerBackend declaration
 *  
 * 2005-06-07
 * Olivier Delannoy
 *
 *
 */
#ifndef YML_CORE_BACKEND_HH
#define YML_CORE_BACKEND_HH 1
#include "core.hh"
namespace Yml {
namespace Core {
class SchedulerTask;
/**
 * @brief a Backend is a middleware adaptor. 
 * 
 * This class defines the interface between a middleware and
 * the YML environment. It defines an extension interface for the YML Middleware 
 * 
 */
class Backend
{
protected:
    Backend();
public:
    /**
     * Destructor 
     */
    virtual ~Backend(void);
    /**
     * Initialize the backend module.
     * You should refer to the documentation of the module in order to know the format of initData.
     * 
     * @param initData the name of the configuration file for this backend
     *
     * @return true if the initialization successed false otherwise 
     */ 
    void init(const string& initData);
    /** 
     * Cleanup the backend module. 
     */
    void cleanup();
    /**
     * Execute a task on the middleware 
     *
     * @return true on success and set uid of the task  
     */
    bool execute(SchedulerTask* task);
    /**
     * Retrieve a finished task 
     *
     * @return a pointer to the SchedulerTask object corresponding to one of the finished task. 
     */
    SchedulerTask* retrieve(void);
    /**
     * Cancel a task 
     *
     */
    void cancel(uint32 appId, uint32 taskId);
protected:
    virtual void initImpl(const string& initData) = 0;
    virtual void cleanupImpl() {}
    virtual bool executeImpl(SchedulerTask* task) = 0;
    virtual SchedulerTask* retrieveImpl(void) = 0;
    virtual void cancelImpl(uint32 appId, uint32 taskId)  {}

    string mDRHost;
    string mDRPort;
    string mBinGroup;
    
private:
    /** Disabled copy ctor */
    Backend(Backend& src);
    /** Disabled assignement operator */
    Backend& operator=(Backend& src);
}; 
}
}
#ifdef HAVE_INLINE
#include "Backend.icc" 
#endif 
#endif

