/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class BackendManager function definition 
 *
 * 2005-08-21
 * Olivier Delannoy
 */
#include "BackendManager.hh"
#include "PluginManager.hh" 
#include "Backend.hh"
#include "SecondLevelScheduler.hh" 
#include "SchedulerTask.hh" 
#include <algorithm>
#include <LoggerFactory.hh>

namespace Yml
{
namespace Core
{
bool BackendManager::SecondLevelSchedulerFunctor::operator()(SchedulerTask* const& left, SchedulerTask* const& right)
{
    return mScheduler.lessUrgent(left, right);
}


    

BackendManager::BackendManager(void) 
    : mBackend(PluginManager::getSingleton().getBackend()), 
      mSched(PluginManager::getSingleton().getSecondLevelScheduler())
{
}


BackendManager::~BackendManager(void)
{
    mBackend.cleanup();
}

SchedulerTask* BackendManager::finishedTask(void) 
{
    SchedulerTask* res = 0;
    //Retrive all tasks from the backends 
    while((res = mBackend.retrieve()) != 0)
        mFinished.push_back(res);
    res = 0;
    if (mFinished.size())
    {
        res = mFinished.front();
        mFinished.pop_front();
    }
    return res;
}


void BackendManager::queue(SchedulerTask* task)
{
    if (task->isCall())
    {
        mQueued.push_back(task);
    }
    else // Builtins and signals 
    {
        mFinished.push_back(task);
    }
}

/*
struct Prioritizer 
{
    bool operator()(SchedulerTask* const& t1, SchedulerTask* const& t2) 
    { 
        return t2->policy() < t1->policy();
    }
};
*/

bool BackendManager::schedulePendingTasks(void)
{
    
    // second level scheduler 
    // The second level scheduler order tasks ready for scheduling 
    // The first task will be launch first It relies on the sort 
    // algorithm of the standard template library which accept
    //UTIL_DEBUG("default", "Order task to be sent to the backend");
    //std::sort(mQueued.begin(), mQueued.end(), mSched);
    UTIL_DEBUG("default", "Launching " << mQueued.size() << " tasks");
    const std::vector<SchedulerTask*>::reverse_iterator end = mQueued.rend();
    std::vector<SchedulerTask*>::reverse_iterator iter = mQueued.rbegin();

    for (; iter != end ; ++iter)
    {
        if (! mBackend.execute(*iter))
        {
            UTIL_ERROR("default", "Scheduling task " <<(*iter)->index() << ": an error occured");
            return false;
        }
    }
    mQueued.clear();
    return true;
}
}
}



