/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class BackendManager declaration
 *  
 * 2005-08-19
 * Olivier Delannoy
 */
#ifndef YML_CORE_BACKENDMANAGER_HH
#define YML_CORE_BACKENDMANAGER_HH 1
#include "core.hh"
#include <Singleton.hh>
#include <vector>
#include <deque>


namespace Yml {
namespace Core {
class SchedulerTask;
class Backend;
class SecondLevelScheduler;


/**
 * @brief Handle all interaction between the YML Framework and the backends. 
 *
 * This object interact with registered backend. There is only
 * one object to deals with backend interaction. It's roles consists in loading, 
 * unloding, querying existing backends. 
 */
class BackendManager : public Util::Singleton<BackendManager>
{
public:
    /**
     * Default Contstructor
     */
    BackendManager(void);
    /**
     * Destructor
     */
    ~BackendManager(void);
    /** 
     * Retrieve executed task 
     */
    SchedulerTask* finishedTask(void);
    /**
     * Queue a task ready for execution 
     */ 
    void queue(SchedulerTask* task = 0);
    /** 
     * Start execution of all queued tasks 
     */ 
    bool schedulePendingTasks(void);
    
    using Util::Singleton<BackendManager>::getSingleton;
    using Util::Singleton<BackendManager>::getSingletonPtr;
protected:
    
private:
    /*! 
      \brief 
      Functor adapter to the Second level scheduler interface for the sort algorithm.
      
      Functor adapter to the Second level scheduler interface for the sort algorithm.
      
      \internal 
    */
    class SecondLevelSchedulerFunctor
    {
    public:
        SecondLevelSchedulerFunctor(SecondLevelScheduler& scheduler) : mScheduler(scheduler)  {}
        bool operator()(SchedulerTask* const& left, SchedulerTask* const& right);
    private:
        SecondLevelScheduler& mScheduler;
    };
    

    typedef std::deque<SchedulerTask*> TaskQueue;
    std::vector<SchedulerTask*> mQueued;
    TaskQueue mFinished;
    Backend& mBackend;
    SecondLevelSchedulerFunctor mSched;
    
};

}
}
#endif

