/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class Component_xmlHandler declaration
 *  
 * 2005-08-21
 * Olivier Delannoy
 */
#ifndef YML_CORE_COMPONENT_XMLHANDLER_HH
#define YML_CORE_COMPONENT_XMLHANDLER_HH 1
#include "core.hh"
#include "ComponentAbstract.hh"
#include "ComponentGraph.hh"
#include "ComponentImpl.hh" 

#include <XMLHandler.hh> 

#include <cassert> 
namespace Yml 
{
namespace Core 
{
/*! 
  \brief 
  XMLHandler for a component definition query
  
  This handler is used to parse XML Component based query.  It can
  parse all kind of component. The result can be selected using
  getComponentType() method. The associated getComponentXXX method
  provides the resulting component. 
  
*/
class Component_xmlHandler : public Util::XMLHandler 
{
public:
    /*! 
      \brief 
      Default ctor
      
      Default ctor
    */
    Component_xmlHandler();
    ~Component_xmlHandler();
    
    /*! 
      Return the component type of the parsed object.
      - "": an error occured 
      - "abstract": it's an abstract component 
      - "graph": it's a graph component 
      - "impl": it's  an implementation component 
    */
    ComponentType getComponentType() const;
    

    /*! 
      Return the abstract component parsed
      \note this fonction is valid only if getComponentType() return COMPONENT_TYPE_ABSTRACT.
    */
    const ComponentAbstract& getComponentAbstract() const;
    /*! 
      Return the graph component parsed
      \note this fonction is valid only if getComponentType() return COMPONENT_TYPE_GRAPH.
    */   
    const ComponentGraph& getComponentGraph() const;
     /*! 
       Return the implementation component parsed
      \note this fonction is valid only if getComponentType() return COMPONENT_TYPE_IMPL. 
    */   
    const ComponentImpl& getComponentImpl() const;
    /*! 
      OpenTag handler
    */
    void elementStart(const string& element, const Util::Properties& attrs);
    /*! 
      Tag close end handler
    */
    void elementEnd(const string& element);
    /*! 
      Text handler
    */
    void text(const string& text);
private:
    ComponentAbstract* mComponentAbstract;
    ComponentGraph* mComponentGraph;
    ComponentImpl* mComponentImpl;
    ComponentType mComponentType;
    ParameterList_xmlHandler* mParameterList_xmlHandler;
    DistributeList_xmlHandler* mDistributeList_xmlHandler;
    bool mError;
    bool mInHeader;
    bool mInSource;
    bool mInFooter;
};   
}
}
#ifdef HAVE_INLINE
#include "Component_xmlHandler.icc"
#endif 
#endif

