/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class DevelopmentCatalog function definition 
 *
 * 2005-06-14
 * Olivier Delannoy
 */
#include "DevelopmentCatalog.hh"
#ifndef HAVE_INLINE
#define inline
#include "DevelopmentCatalog.icc"
#undef inline
#endif 
#ifdef HAVE_CONFIG_H
#include <yml_config.hh>
#endif 
#include "ParameterList.hh"
#include "ComponentAbstract.hh"
#include "ComponentGraph.hh"
#include "ComponentImpl.hh"

#include "DistributeList.hh"
#include <iostream>

namespace Yml
{
namespace Core
{
DevelopmentCatalog::~DevelopmentCatalog(void)
{
	// Clear cache
	Cache::iterator iter = mCache.begin();
	for(; iter != mCache.end() ; ++iter)
	{
		delete iter->second;
	}
}

const ParameterList* DevelopmentCatalog::getComponentScheme(const string& name)
{
	// Look in the online memory cache
	Cache::iterator iter = mCache.find(name);
	if (iter != mCache.end())
	{
		return iter->second;
	}
	else
	{
		ComponentAbstract const* cmp = retrieveComponentAbstract(name);
		if (cmp)
		{
			mCache[name] = new ParameterList(cmp->getParameterList());
			release(cmp);
			return mCache[name];
		}
	}
	return 0;
}

int DevelopmentCatalog::getImplComponentNbNodes(const string& name)
{
     int nprocs;
     ComponentImpl const* cmp = retrieveComponentImpl(name);
     if(cmp){
        if(strcmp("XMP",cmp->getImplLanguage().c_str())==0)
            nprocs = cmp->getImplNbNodes();
        else
            nprocs = 1;
        release(cmp);
        return nprocs;
     }
     return 0;
}

void DevelopmentCatalog::setComponentScheme(const string& name, const DistributeList& distList)
{
	// Look in the online memory cache
	Cache::iterator iter = mCache.find(name);
	if (iter != mCache.end())
	{

	}

	ParameterList * paramList = mCache[name];

	for(size_t i = 0; i < distList.count(); i++)
	{
		const string& distribName = distList.get(i).getName();
		size_t j = 0;

		while(j < paramList->count() && paramList->get(j).getName() != distribName ) j++;

		paramList->get(j).setMapping(distList.get(i).getMapping());
		paramList->get(j).setDataAlign(distList.get(i).getDataAlign());
		paramList->get(j).setTemplateAlign(distList.get(i).getTemplateAlign());
		paramList->get(j).setSize(distList.get(i).getSize());

	}


}
}
}
