/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/
/**
 * @file
 * @brief Class ParameterList function definition 
 *
 * 2007-01-10
 * Olivier Delannoy
 */
#include "ParameterList.hh"
#ifndef HAVE_INLINE
#define inline
#include "ParameterList.icc"
#undef inline
#endif
#ifdef HAVE_CONFIG_H
#include <yml_config.hh> 
#endif 
namespace Yml 
{
namespace Core
{
Util::XMLSerializer& ParameterList::toXML(Util::XMLSerializer& stream) const
{
    stream.openTag("params");
    std::vector<Parameter>::const_iterator end = mParameters.end();
    std::vector<Parameter>::const_iterator iter = mParameters.begin();
    for (;iter != end ; ++iter)
    {
        stream.openTag("param")
            .attribute("name", iter->getName())
            .attribute("type", iter->getType()) 
            .attribute("description", iter->getDescription())
            .attribute("collection", (iter->isCollection() ? "yes" : "no"));
        switch(iter->getIOMode())
        {
        case PARAM_IO_IN:
            stream.attribute("mode", "in");
            break;
        case PARAM_IO_OUT:
            stream.attribute("mode", "out");
            break;
        case PARAM_IO_INOUT:
            stream.attribute("mode", "inout");
            break;
        }
        stream.closeTag();
    }
    stream.closeTag();
    return stream;
}
    
Util::CSVSerializer& ParameterList::toCSV(Util::CSVSerializer& stream) const
{
    std::vector<Parameter>::const_iterator end = mParameters.end();
    std::vector<Parameter>::const_iterator iter = mParameters.begin();
    for (;iter != end ; ++iter)
    {
        stream.newRow()
            .cell(iter->getName())
            .cell(iter->getType())
            .cell(iter->isCollection() ? "yes" : "no");
        switch(iter->getIOMode())
        {
        case PARAM_IO_IN:
            stream.cell("in");
            break;
        case PARAM_IO_OUT:
            stream.cell("out");
            break;
        case PARAM_IO_INOUT:
            stream.cell("inout");
            break;
        }
        stream.cell(iter->getDescription());
    }
    return stream;
}

ParameterList_csvHandler::ParameterList_csvHandler(ParameterList& list) 
    : mList(list), mCellId(0)
{
}

void ParameterList_csvHandler::rowStart()
{
    mList.add(Parameter());
    mCellId = 0;
}

void ParameterList_csvHandler::cell(const string& value)
{
    switch(mCellId)
    {
    case 0: // Value 
        mList.mParameters.back().setName(value);
        break;
    case 1: // Type 
        mList.mParameters.back().setType(value);
        break;
    case 2: // Collection 1/0
        mList.mParameters.back().setIsCollection(value == "1");
        break;
    case 3: // Mode In/Out/Inout
        if (value == "in")
        {
            mList.mParameters.back().setIOMode(PARAM_IO_IN);
        }
        else if (value == "out")
        {
            mList.mParameters.back().setIOMode(PARAM_IO_OUT);
        }
        else 
        {
            mList.mParameters.back().setIOMode(PARAM_IO_INOUT);
        }
        break;
    case 4:
        mList.mParameters.back().setDescription(value);
        break;
    }
    ++mCellId;    
}

ParameterList_xmlHandler::ParameterList_xmlHandler(ParameterList& list)
    : mList(list)
{ 
}

bool ParameterList_xmlHandler::isHandled(const string& element) 
{
    if (element == "params" || element == "param")
        return true;
    return false;
}
void ParameterList_xmlHandler::elementStart(const string& element, const Util::Properties& attributes)
{
    if (element == "param")
    {
        mList.mParameters.push_back(Parameter());
        Parameter& ref = mList.mParameters.back();
        ref.setName(attributes.stringValue("name"));
        ref.setType(attributes.stringValue("type"));
        ref.setDescription(attributes.stringValue("description"));
        const string& mode = attributes.stringValue("mode", "inout");
        if (mode == "in") 
            ref.setIOMode(PARAM_IO_IN);
        else if (mode == "out")
            ref.setIOMode(PARAM_IO_OUT);
        else 
            ref.setIOMode(PARAM_IO_INOUT);
        ref.setIsCollection(attributes.booleanValue("collection", false));
    }
}

}
}
