/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/
/**
 * @file
 * @brief Class ParserError declaration
 *  
 * 2005-12-27
 * Olivier Delannoy
 */
#ifndef YML_CORE_PARSERERROR_HH
#define YML_CORE_PARSERERROR_HH 1
#include "core.hh"
namespace Yml {
    namespace Core {
        /**
         * @brief This class is used to create a list of error 
         * detected during the parsing stage. 
         *
         * This class is used to help the user to track error 
         * in its program. When it's possible error message 
         * are associated to a location. 
         */
        class ParserError
        {
        public:
            /** 
             * Constructor 
             * @param message the text of the error 
             * @param line the line number where the error occured 
             * @param column the column number of the error  
             */ 
            ParserError(const string& message, uint32 line = 0, uint32 column = 0);
            /** 
             * Get the text of the error 
             */
            const string& message(void) const;
            /** 
             * Get the line of the error 
             */
            uint32 line(void) const;
            /** 
             * Get the column of the error 
             */
            uint32 column(void) const;
            /** 
             * Return a String corresponding to the error
             * The format is simple: 
             * line:column:error message 
             */
            string toString(void) const;
        protected:
    
        private:
            /** Store the message associated to the error */
            string mMessage;
            /** Store the line associated to the error */
            uint32 mLine;
            /** Store the column of the error */
            uint32 mColumn;
        };
    }
}
#ifdef HAVE_INLINE
#include "ParserError.icc"
#endif
#endif
