/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class PreCondition function definition 
 *
 * 2005-08-04
 * Olivier Delannoy
 */
#include "PreCondition.hh"
#ifndef HAVE_INLINE
#define inline
#include "PreCondition.icc"
#undef inline
#endif 
#include <cassert> 
Yml::Core::PreCondition::PreCondition(const Yml::Core::string& eventName)
    : mOperator(OP_EVENT),
      mEventName(eventName), 
      mLeft(0), 
      mRight(0)
{
}


Yml::Core::PreCondition::PreCondition(Yml::Core::uint32 op, 
                                      Yml::Core::PreCondition* left, 
                                      Yml::Core::PreCondition* right)
    : mOperator(op), 
      mLeft(left), 
      mRight(right)
{
    assert(op >=OP_AND && op <= OP_XOR);
    assert(left);
    assert(right);
}


Yml::Core::PreCondition::PreCondition(Yml::Core::PreCondition* negateMe)
    : mOperator(OP_NOT), 
      mLeft(negateMe), 
      mRight(0)
{
    assert(negateMe);    
}


Yml::Core::PreCondition::~PreCondition(void)
{
    delete mLeft;
    delete mRight;
}


Yml::Core::PreCondition* Yml::Core::PreCondition::clone(void) const 
{
    switch(mOperator)
    {
    case OP_EVENT:
        return new PreCondition(mEventName);
    case OP_AND:
        return new PreCondition(OP_AND, mLeft->clone(), mRight->clone());
    case OP_OR:
        return new PreCondition(OP_OR, mLeft->clone(), mRight->clone());
    case OP_XOR:
        return new PreCondition(OP_XOR, mLeft->clone(), mRight->clone());
        
    case OP_NOT:
        return new PreCondition(mLeft->clone());
    default:
        return 0;
    }
    
}

void Yml::Core::PreCondition::print(std::ostream& out) const 
{
    out << "(";
    switch(mOperator)
    {
    case OP_EVENT:
        out << mEventName;
        break;
        
    case OP_AND:
        out << "AND ";
        mLeft->print(out);
        mRight->print(out);
        break;

    case OP_OR:
        out << "OR ";
        mLeft->print(out);
        mRight->print(out);
        break;

    case OP_XOR:
        out << "XOR ";
        mLeft->print(out);
        mRight->print(out);
        break;
        
    case OP_NOT:
        out << "NOT ";
        mLeft->print(out);
        break;        
    }
    out << ")";
    
}

