/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class SchedulerTask function definition 
 *
 * 2005-08-24
 * Olivier DELANNOY
 */
#include "SchedulerTask.hh"
#ifndef UTIL_COMPAT_HAVE_INLINE
#define inline
#include "SchedulerTask.icc"
#undef inline
#endif 
#include <Pack.hh>
#include <fstream>
#include <RuntimeEnvironment.hh>
#include <Dir.hh>
#include <File.hh>
namespace Yml
{
namespace Core
{

SchedulerTask::SchedulerTask(uint32 index)
    : mIndex(index), mAppId(0), mStatus(ExecSuccess)
{
}

SchedulerTask::~SchedulerTask(void)
{
    
}

void SchedulerTask::policy(SchedulingPolicy policy)
{
    mPolicy = policy;
}

void SchedulerTask::uid(const string& uid)
{
    mUid = uid;
}
const string& SchedulerTask::uid() const 
{
    return mUid;
}
const string& SchedulerTask::workDescription() const
{
    return mWorkDescription;
}
void SchedulerTask::setPackNames(const string& packIn,  const string& packOut, const string& packtmp)
{
    mInputPackFile = packIn   + mUid + "_packin.pack";
    mOutputPackFile = packOut + mUid + "_packout.pack";
    mAdminPackFile = packOut   + mUid + "_packadmin.pack";
    mWorkDescription = packIn + mUid + "_work.xml";
    mWorkDir = packtmp;
}


bool SchedulerTask::extractAdminInfo(void)
{
    if (isBuiltinCall())
        return true;
    mStatus = ExecSuccess;
    
    Util::Pack pack(mAdminPackFile);
    Util::Dir::mkdir(mAdminPackFile + "_files");
    pack.extractAll(mAdminPackFile + "_files");
    pack.close();
    // Parse status 
    string tmp;
    std::ifstream istatus((mAdminPackFile + "_files" + Util::RuntimeEnvironment::PATH_SEPARATOR + "status.txt").c_str());
    if (istatus)
    {
        istatus >>tmp;
        if (tmp.empty())
        {
            mStatus = ExecError;
        }
        if (tmp == "success")
        {
            mStatus = ExecSuccess;
        }
        else if (tmp == "error")
        {
            mStatus = ExecError;
        }
        else if (tmp == "exception")
        {
            mStatus = ExecException;
            istatus >> mExceptionName;
        }
        else 
        {
            mStatus = ExecError;
        }
        istatus.close();
    }
    else 
    {
        mStatus = ExecError;
    }
    std::ifstream itrace((mAdminPackFile + "_files" + Util::RuntimeEnvironment::PATH_SEPARATOR + "trace.txt").c_str());
    while(itrace && !itrace.eof())
    {
        std::getline(itrace, tmp);
        mTrace += tmp + '\n';
    }
    itrace.close();
    Util::Dir::rmdir(mAdminPackFile + "_files");
    return true;
}

}
}

