/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief SecondLevelScheduler declaration
 *  
 * 2005-08-26
 * Olivier DELANNOY
 */
#ifndef YML_CORE_SECONDLEVELSCHEDULER_HH
#define YML_CORE_SECONDLEVELSCHEDULER_HH 1
#include "core.hh"
namespace Yml {
namespace Core {
class SchedulerTask;
/**
 * @brief Abstract interface for the second level scheduler used in the backend manager.
 *
 * This defines the abstract interface of a
 * SecondLevelScheduler object which mainly consists in a
 * functor. The aim of this object consist in providing an
 * ordering function for two SchedulerTask pointer. The
 * ordering is as follow : The first task in the sequence will
 * be scheduled first.  It means that the task with the lowest
 * priority must be at last in the sequence.  The main
 * function in this class is a simple comparaison function
 * which return true if the first element have a greater
 * priority than the second one.
 * 
 *
 * Implementations must defines only one function : 
 * bool operator(const SchedulerTask*& x, const SchedulerTask*& y);
 * This function return true if x must be launch before y and false otherwise. 
 *
 */
class SecondLevelScheduler
{
public:
    virtual ~SecondLevelScheduler() {}
    /** 
     * Return true if task1 is less urgent than task2
     */
    virtual bool lessUrgent(SchedulerTask*const & task1, SchedulerTask* const& task2) = 0;
    virtual void init(const string& initData) = 0;
    
    
};

}
}
#endif

