/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class Task function definition 
 *
 * 2005-07-25
 * Olivier Delannoy
 */
#include "Task.hh"
#ifndef HAVE_INLINE
#define inline
#include "Task.icc"
#undef inline
#endif 
#include <TaskComputation.hh>
// Notify Task is the default 
Yml::Core::Task::Task(Yml::Core::YvetteContext& context)
    : mComputation(0),
      mPreCondition(context.preCondition()), 
      mPostEvents(context.post())
{
    context.resetPre();
    context.resetPost();
}

Yml::Core::Task::~Task(void)
{
    delete mComputation;
    delete mPreCondition;
}

static std::ostream& operator<<(std::ostream& out, const std::vector<Yml::Core::string>& obj)
{
    std::vector<Yml::Core::string>::const_iterator iter = obj.begin();
    for( ; iter != obj.end() ; ++iter)
    {
        out << " " << *iter;
    }
    return out;
}


void Yml::Core::Task::print(std::ostream& out) const
{
    out << "*******************************************" << std::endl;
    if (mComputation)
    {
        // Component Call 
        out << "Component Call Task" << std::endl;
        out << "*******************************************" << std::endl;
        out << "Condition  ....: " << *mPreCondition << std::endl;
        out << "Events ........:" << mPostEvents << std::endl;
    }
    else 
    {
        // Notify Task
        out << "NotifyTask" << std::endl;
        out << "*******************************************" << std::endl;
        out << "Condition  ....: " << *mPreCondition << std::endl;
        out << "Events ........: " << mPostEvents << std::endl;
    }
    out << "*******************************************" << std::endl;
}



