/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class YvetteCompiler function definition 
 *
 * 2005-06-18
 * Olivier Delannoy
 */
#include "YvetteCompiler.hh"
#ifndef HAVE_INLINE
#define inline
#include "YvetteCompiler.icc"
#undef inline
#endif 
#include "AST.hh"
#include "YvetteParserDriver.hh"
#include "Application.hh"
#include "YvetteTaskGenerator.hh"

// Logging and message output 
#include <Application.hh>
#include <LoggerFactory.hh>
 
Yml::Core::YvetteCompiler::YvetteCompiler(void)
    :  mApplication(0), mProgressEventIndex(0), 
       mPrefix("") // Default prefix for a top level compilation graph compilation will require this ...
{
    
}

Yml::Core::YvetteCompiler::~YvetteCompiler(void)
{
    
}



bool Yml::Core::YvetteCompiler::compile(void)
{
    UTIL_INFO("default", "Start compiling process");
    mSymbols.pushContext();
#ifdef HAVE_DEBUG    
    Yml::Core::YvetteParserDriver parser(true, true);
#else 
    Yml::Core::YvetteParserDriver parser(false, false);
#endif  
    Yml::Core::AST *ast = parser.parse(mSource);

    if (ast == 0) 
    {
        ErrorList::const_iterator iter;
        ErrorList::const_iterator end = parser.errors().end() ;
        for(iter = parser.errors().begin() ; iter != end ; ++iter)
        {
          UTIL_ERROR("default", iter->toString());
        }
        // There's a syntax error ast->printTree(UTIL_OUT());
        // Generate the report 
        UTIL_ERROR("default", "Syntax  error detected");
	return false;
    }    
    UTIL_INFO("default", "Parsing stage finished");
#ifdef HAVE_DEBUG
    ast->printTree(std::cout); 
#endif 
    // Go to Tree rewriting step
    YvetteContext context;
    context.preCondition(new PreCondition(mPrefix + "##start"));


    // Create a Program object 
    //    YvetteProgram program(ast); << Future 
    
    


    // Go to Generation step 
    Yml::Core::YvetteTaskGenerator generator(this);
    bool status = generator.generate(ast->getFirstChild(), context);
    if (! status)
    {
        //oups its an error 
        UTIL_ERROR("default", "Application generation failed");
        return false;
    }
    UTIL_INFO("default", "Application Generation successfull");
    // Work Done ! ! !
    return true;
}

