/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class YvetteCompiler declaration
 *  
 * 2005-06-18
 * Olivier Delannoy
 */
#ifndef YML_CORE_YVETTECOMPILER_HH
# define YML_CORE_YVETTECOMPILER_HH 1
# include "core.hh"
#include <vector>
#include "YvetteSymbolTable.hh"
#include "YvetteApplication.hh" 
namespace Yml {
    namespace Core {
        class CompilerMessage;
        /**
         * @brief Yvette Compiler Class
         * 
         * The goal of this class is to create a compiler for an
         * application and in a near futur graphs. The standard
         * behaviour is as follow, one create an YvetteCompiler
         * object, set the application where to add events, exceptions
         * and tasks, set the compiler initial symbols, set the
         * application source and finally launch the compilation
         * process.
         */
        class YvetteCompiler
        {
        public:
            /**
             * Default Contstructor
             */
            YvetteCompiler(void);
            /**
             * Destructor
             */
            ~YvetteCompiler(void);
            /** 
             * Set the application object target of the compilation.
             */
            void setApplication(Application* application);
            /** 
             * Set the source object target of the compilation and the initial line 
             */
            void setSource(const string& source);
            /**
             * Set the first line 
             */
            void setFirstLine(uint32 mFirstLine);
            
            /** Access the symbol table  */
            YvetteSymbolTable& getSymbolTable(void);
            /** Access the application object */
            Application& getApplication(void);
            
            /** Start compilation */ 
            bool compile(void);
            /** Get the source code to compile */
            const string& getSource(void) const;
            /** Get the line  of the source code */
            uint32 getFirstLine(void) const;
            /** Add a compilation message */
            void addMessage(CompilerMessage* message);
            /** Get an event name unique in this compilation stage */
            string generateProgressEvent(void);
            /** Get event prefix */
            const string& eventPrefix(void) const;
        protected:
            
        private:
            /** Store the application */
            Application *mApplication;
            /** Store the source */ 
            string mSource;
            /** Store the first line */
            uint32 mLine;
            /** Store all symbol */
            YvetteSymbolTable mSymbols;
            /** Store the number of progressEventRequest */
            uint32 mProgressEventIndex;
            /** prefix for all events */
            string mPrefix;
            /** Store all message */
            std::vector<CompilerMessage*> mMessages;
        };
    }
}
# ifdef HAVE_INLINE
#  include "YvetteCompiler.icc"
# endif
#endif

