/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class YvetteFunction declaration
 *  
 * 2005-06-28
 * Olivier Delannoy
 */
#ifndef YML_CORE_YVETTEFUNCTION_HH
#define YML_CORE_YVETTEFUNCTION_HH 1
#include "core.hh"
#include "YvetteObject.hh"
#include <vector> 

namespace Yml {
    namespace Core {

        /**
         * @brief Base class of all function. It's a generic interface
         * for handling any function.
         * 
         * This class represents all function available in the
         * language. Each object of this class can be used to handle
         * multiple function query, one at a time. Each user function
         * return an YvetteObject. There is no side effect. A function
         * can not update it's parameters. 
         * 
         * In order to be references in the Compiler each function
         * must be registered before its usage @see Yml::Core::YvetteFuntionTable
         */
        class YvetteFunction
        {
        public:
            virtual ~YvetteFunction(void) {}
            /** 
             * Register a parameter to the YvetteFunction 
             */
            void addParameter(const YvetteObject& parameter);
            /**
             * execute the function and return the result 
             *
             * @return the resulting object or
             * Yml::Core::YvetteObject::INVALID_OBJECT if an error
             * occured
             */
            YvetteObject execute(void);
            
        protected:
            /**
             * All function class must provide an overloaded version
             * of this function. It is the real implementation of execute(). 
             * 
             * @return the resulting object or
             * Yml::Core::YvetteObject::INVALID_OBJECT if an error
             * occured
             */
            virtual YvetteObject executeImpl(void) = 0;
            
            /** 
             * Store function parameters until its execution 
             *
             * @note the method execute automatically reset this vector 
             */
            
            std::vector<YvetteObject> mParameters;
        private:
            
        };
    }
}
#ifdef HAVE_INLINE
#include "YvetteFunction.icc"
#endif
#endif

