/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class YvetteFunctionTable function definition 
 *
 * 2005-06-28
 * Olivier Delannoy
 */
#include "YvetteFunctionTable.hh"
#ifndef HAVE_INLINE
#define inline
#include "YvetteFunctionTable.icc"
#undef inline
#endif 
#include "YvetteFunction.hh" 
#include "YvetteFunctionPackage.hh"
#include <Plugin.hh>
#include <PluginFactory.hh>
#include <LoggerFactory.hh>
#include <Application.hh>

Yml::Core::YvetteFunctionTable::YvetteFunctionTable(void)
{
}


Yml::Core::YvetteFunctionTable::~YvetteFunctionTable(void)
{
    // Free all function 
    
    
}

void Yml::Core::YvetteFunctionTable::loadPackage(const Yml::Core::string& name)
{
    UTIL_DEBUG("default", "Loading functions package '" << name << "'");
    
    // Check whether package exists or not 
    if (mFunctions.find(name) == mFunctions.end())
    {
        // Load it from a Plugin 
        Util::Plugin* plug = Util::PluginFactory::getSingleton().create("Function_" + name);
        if (plug && plug->isOpened())
        {
            // Resolve create symbol Yml::Core::YvetteFunctionPackage* (*func)(void)
            YvetteFunctionPackage* (*func)(void) = (YvetteFunctionPackage* (*)(void))plug->loadSymbol("create");
            if (func)
            {
                YvetteFunctionPackage* package = func();
                if (package)
                {
                    for(uint32 i = 0 ; i < package->size() ; ++i)
                    {
                        registerFunction(name, package->getName(i), package->getFunction(i));
                    }
                    UTIL_INFO("default", "Loaded functions package '" << name << "' successfully, " << package->size() << " functions loaded.");
                    delete package;
                }
                else 
                {
                    UTIL_ERROR("default", "Loading functions package '" << name << "' failed, No package object found");
                }
            }
            else
            {
                UTIL_ERROR("default", "Loading functions package '" << name << "' failed while resolving symbol 'create': " << plug->lastError());
            }
        }
        else
        {
            UTIL_ERROR("default", "Loading functions package '" << name << "' failed on plugin loading: " << plug->lastError());
        }
    }
    else 
    {
        UTIL_DEBUG("default", "Loading of functions package '" << name << "' already done, skipping");
    }
}





