/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class YvetteFunctionTable declaration
 *  
 * 2005-06-28
 * Olivier Delannoy
 */
#ifndef YML_CORE_YVETTEFUNCTIONTABLE_HH
#define YML_CORE_YVETTEFUNCTIONTABLE_HH 1
#include "core.hh"
#include <map>
#include <Singleton.hh>
#include <LoggerFactory.hh>

namespace Yml {
    namespace Core {
        class YvetteFunction;
        
        /**
         * @brief This class Store all function available at a time.
         *
         * This class Deal with all function request. A function is an
         * object that get some YvetteObjects and generate a resulting
         * YvetteObject. The YvetteFunction class represents available
         * function and are all registered in this table. This is a
         * two level map of string and YvetteFunction. YvetteFunction
         * are used during the graph compilation. They are organised
         * in namespace the not named namespace is always present and
         * represents builtin features. A function does not store any
         * state two execution of the function with the same set of
         * YmlObject in input should generate the same results. 
         */
        class YvetteFunctionTable : public Util::Singleton<YvetteFunctionTable>
        {
        public:
            /**
             * Default constructor 
             */
            YvetteFunctionTable(void);
            /**
             * Destructor 
             */
            virtual ~YvetteFunctionTable(void);
            
            /** 
             * Get the YvetteFunction object associated to the tuple 
             * package / function.
             * The default package or namespace is named "" and contains 
             * only default function. 
             *
             * @param package the name of the package.
             * @param function the name of the function to call.
             * @return a pointer to a function object or 0 if not found. 
             */
            YvetteFunction* get(const string& package, const string& function);
            /** 
             * load a new package of function and update function list. 
             *
             * @param package the name of the package.
             */
            void loadPackage(const string& package);
            

            using Util::Singleton<YvetteFunctionTable>::getSingleton;
            
            using Util::Singleton<YvetteFunctionTable>::getSingletonPtr;
            
        protected:
            /**
             * Register a function 
             *
             * @param package the name of the package where to add the functions  
             * @param function the name of the function to call. 
             * @param functor a pointer to a function object. 
             */
            void registerFunction(const string& package, const string& function, YvetteFunction* functor);
            
        private:
            /** Store all function available */ 
            std::map<string, std::map<string, YvetteFunction*> > mFunctions;
            
        };
    }
}
#ifdef HAVE_INLINE
#include "YvetteFunctionTable.icc"
#endif
#endif

