/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class YvetteObject declaration
 *  
 * 2005-06-28
 * Olivier Delannoy
 */
#ifndef YML_CORE_YVETTEOBJECT_HH
#define YML_CORE_YVETTEOBJECT_HH 1
#include "core.hh"
#include <iostream>
namespace Yml {
    namespace Core {
        /**
         * @brief YvetteObject are involved in the compilation process. 
         *
         * It's a mutable object that is used to store values in the
         * Yvette Compiler. Each object as a type and can be a result
         * or a parameter of a function, a component call, and many
         * other place. It's quite similar to an rvalue in the grammar
         * of the language
         */
        class YvetteObject
        {
        public:
            /** Default constructor */ 
            YvetteObject(void);

            /**
             * Create an YvetteObject of type integer 
             */
            explicit YvetteObject(int32 value);
            /**
             * Create an YvetteObject based on a real 
             */
            explicit YvetteObject(real64 value);
            /**
             * Create an YvetteObject based on a string 
             */
            explicit YvetteObject(const string& value);
            
            /**
             * Destructor
             */
            ~YvetteObject(void);
            /** 
             * Copy constructor 
             */
            YvetteObject(const YvetteObject& obj);
            /** 
             * Assignement operator
             */
            YvetteObject& operator=(const YvetteObject& obj);
            /**
             * Get the type of the object depending on the type of the
             * object certain function will have no effect at all. 
             */
            uint32 getType(void) const;
            /** 
             * Test whether the object value is a truth value or not 
             */ 
            bool isTrue(void) const;
            /** 
             * Test whether the object is valid or not 
             */
            bool isValid(void) const;
            
            /**
             * Retrieve integer value of the object 
             */
            int32 integerValue(void) const;
            /** Retrieve real value of the object */
            real64 realValue(void) const;
            /** Retrieve string value of the object */
            string stringValue(void) const;
            
            static const uint32 INVALID = 0;
            static const uint32 INTEGER = 1;
            static const uint32 REAL    = 2;
            static const uint32 STRING  = 3;
            static const YvetteObject INVALID_OBJECT;
            
            /**
             * Used for logging 
             */
            void print(std::ostream& out) const;
            
        protected:
            
        private:
            /** 
             * Store the value of the YvetteObject 
             */
            union 
            {
                int32 mInteger;
                real64 mReal;
                string* mString;
            } mData;
            uint32 mType;
        };
    }
}

std::ostream& operator<<(std::ostream& out, const Yml::Core::YvetteObject& obj);


#ifdef HAVE_INLINE
#include "YvetteObject.icc"
#endif
#endif

