/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/
/**
 * @file
 * @brief Class yy::YvetteParser generator (bison-2.1)
 *  
 * 2005-12-27
 * Olivier Delannoy
 */
%skeleton "lalr1.cc"
%define "parser_class_name" "YvetteParser"
%defines
%{

#include "YvetteParserDriver.hh" 
#include <LoggerFactory.hh>

%}
%parse-param { Yml::Core::YvetteParserDriver& driver }
%lex-param   { Yml::Core::YvetteParserDriver& driver }
%locations
%initial-action
{
	@$.begin.filename = @$.end.filename = 0;
}
%debug
%error-verbose
//%printer { debug_stream() << *$$;}
  /* To enable memory deallocation during error recovery use %destructor */ 
//%destructor { delete $$; } 

  /* Basic data type */
%token TK_INTEGER   "integer"
%token TK_REAL      "real"
%token TK_STRING    "string"


  /* Operator */ 
%token TK_OP_ADD        "operator add"
%token TK_OP_MIN        "operator minus"
%token TK_OP_MUL        "operator product"
%token TK_OP_DIV        "operator division" 
%token TK_OP_MOD        "operator modulo" 

  /* Boolean operator */ 
%token TK_OP_BOOL_AND   "boolean operator and"
%token TK_OP_BOOL_OR    "boolean operator or" 
%token TK_OP_BOOL_NOT   "boolean operator not" 
%token TK_OP_BOOL_XOR   "boolean operator xor" 

  /* Comparaison */
%token TK_OP_COMP_EQ   "operator equals to"
%token TK_OP_COMP_NEQ  "operator not equals to" 
%token TK_OP_COMP_LE   "operator lesser or equals to"
%token TK_OP_COMP_GE   "operator greater or equals to"
%token TK_OP_COMP_LT   "operator lesser than"
%token TK_OP_COMP_GT   "operator greater than" 

  /* Identifier */ 
%token TK_IDENTIFIER    "identifier"
%token TK_FUNC          "function"

/* Scheduler policy */ 
%token TK_K_COMPUTE     "Compute scheduling policy"
%token TK_K_MIGRATE     "Migrate scheduling policy"

/* Assigment operator */
%token TK_K_ASSIGN      ":="
/* Namespace resolution */
%token TK_K_NAMESPACE   "Namespace"

/* Par groups */
%token TK_K_PAR         "par"
%token TK_K_PAR_END     "endpar"
%token TK_K_PAR_SEP     "//"

/* par do /seq do group */ 
%token TK_K_SEQ         "seq"
%token TK_K_DO          "do"
%token TK_K_DO_COND_ITERATOR "("
%token TK_K_DO_COND_EXCLUDE  "{"
%token TK_K_DO_END       "enddo"

/* If then else group */
%token TK_K_IF          "if"
%token TK_K_IF_END      "endif"
%token TK_K_IF_THEN     "then"
%token TK_K_IF_ELSE     "else"

/* Event handling */
%token TK_K_NOTIFY      "notify"
%token TK_K_WAIT        "wait"
%token TK_K_EVENT       "event" /* Not used */

/* Exception handling (Not Yet Used) */
%token TK_K_EX_EXCEPTION 
%token TK_K_EX_TRY
%token TK_K_EX_TRY_END
%token TK_K_EX_CATCH
%token TK_K_EX_CATCH_END
%token TK_K_EX_FINALLY
%token TK_K_EX_FINALLY_END
%token TK_EOF 0 "end of file" 

%start start 
%%

start: 
  stmt_list { driver.root().setFirstChild($1); }
;
stmt_list: 
/* EMPTY */ {$$ = 0; }
|
stmt stmt_list { $$ = $1; $1->setNextSibling($2);   }

;

stmt:
  simple_stmt ';' {$$=$1; }
|
  complex_stmt {$$=$1; }
;

simple_stmt:
  assign_stmt {$$=$1;}
|
  evt_wait_stmt {$$=$1;}
| 
  evt_notify_stmt {$$=$1;}
| 
  call_stmt {$$=$1;}
;

complex_stmt:
  par_stmt {$$=$1;}
|
  seq_stmt {$$=$1;}
|
  if_stmt {$$=$1;}
;

assign_stmt: 
  lvalue TK_K_ASSIGN rvalue {$2->setFirstChild($1); $1->setNextSibling($3) ; $$=$2; }
;

evt_wait_stmt:
  TK_K_WAIT '(' evt_cond ')' {$$ = $1; $$->setFirstChild($3);}

evt_cond: 
  evt_cond TK_OP_BOOL_OR evt_cond_not {$$=$2; $2->getFirstChild()->setNextSibling($1); $1->setNextSibling($3);}
|
  evt_cond TK_OP_BOOL_XOR evt_cond_not {$$=$2; $2->getFirstChild()->setNextSibling($1); $1->setNextSibling($3);}
| 
  evt_cond TK_OP_BOOL_AND evt_cond_not {$$=$2; $2->getFirstChild()->setNextSibling($1); $1->setNextSibling($3);}
|
  evt_cond_not {$$=$1;}
;

evt_cond_not:
  TK_OP_BOOL_NOT evt_cond_single {$$=$1; $$->getFirstChild()->setNextSibling($2);}
|
  evt_cond_single {$$=$1;}
;

evt_cond_single:
  '(' evt_cond ')' {$$=$2;}
|
  lvalue { $$=$1;}
;
               
  
evt_notify_stmt:
  TK_K_NOTIFY '(' lvalue lvalue_list ')' { $1->setFirstChild($3); $3->setNextSibling($4); $$ = $1;}
 
call_stmt:
  TK_K_COMPUTE TK_IDENTIFIER '(' param_list ')' {$1->setFirstChild($2); $2->setFirstChild($4) ; $$ = $1;}
|
  TK_K_MIGRATE TK_IDENTIFIER '(' param_list ')' {$1->setFirstChild($2); $2->setFirstChild($4) ; $$ = $1;}
;
  /* Parallel, parallel do and sequential do block */
par_stmt:
  TK_K_PAR par_block_list TK_K_PAR_END {$1->setFirstChild($2); $$ = $1; }
| 
  TK_K_PAR pardo_block {$1->setFirstChild($2); $$=$1;}
;  

seq_stmt:
  TK_K_SEQ seqdo_block {$1->setFirstChild($2); $$=$1;}
;

par_block_list:
  stmt_list par_block_list2 {$$ = driver.newASTNode();
                             $$->setType(TK_K_PAR_SEP);
                             $$->setText("//");
                             $$->setLine($1->getLine());
                             $$->setFirstChild($1);
                             $$->setNextSibling($2);}
;
par_block_list2:
  /* EMPTY */ {$$=0;}
|
  TK_K_PAR_SEP stmt_list par_block_list2 { $1->setFirstChild($2); $1->setNextSibling($3); $$= $1;}
;


  /* Quite hard rule because we reorder nodes from the instruction 
   * The do_cond_exclude_list is transformed into an if with an and between all expression
   * The do_cond_list is translated into imbricate par do statement with one 
   * iterator per level We first compute the if with statement list and then we does 
   * the expansion of the iterators 
   */
pardo_block:
  do_cond_iterator_list do_cond_exclude_list TK_K_DO stmt_list TK_K_DO_END 
  {
   /* $1 -> iterators 
      $2 -> exclude 
      $3 -> do 
      $4 -> stmts 
    */
    /* Handle exclude list */ 
    Yml::Core::AST* stmt  = $4;
    Yml::Core::AST* doast = 0;
    Yml::Core::AST* iter  = 0;
    if ($2 != 0)
    {
      /* Create the if stmt arround $4 */
      Yml::Core::AST * tmp = $2;
      Yml::Core::AST * thenast = 0;
      Yml::Core::AST * elseast = 0;
      Yml::Core::AST * left = 0;
      Yml::Core::AST * andast = 0;
      Yml::Core::AST * funcast = 0;
      stmt = driver.newASTNode();
      stmt->setType(TK_K_IF);
      stmt->setText("if");
      stmt->setLine($2->getLine());
      stmt->setFirstChild(tmp);
      while (tmp->getNextSibling())
      {
        left = tmp;
        tmp = tmp->getNextSibling();
        left->setNextSibling(0);
        left = stmt->getFirstChild();
        funcast = driver.newASTNode();
        funcast->setType(TK_FUNC);
        funcast->setText("function");
        funcast->setLine(stmt->getLine());
        andast = driver.newASTNode();
        funcast->setFirstChild(driver.newASTNode());
        funcast->getFirstChild()->setText("");
        funcast->getFirstChild()->setType(TK_IDENTIFIER);
        funcast->getFirstChild()->setLine(stmt->getLine());
        funcast->getFirstChild()->setFirstChild(andast);
        andast->setType(TK_IDENTIFIER);
        andast->setText("operator_and");
        funcast->getFirstChild()->setNextSibling(left);
        left->setNextSibling(tmp);
        stmt->setFirstChild(funcast);
      }
      thenast = driver.newASTNode();
      thenast->setType(TK_K_IF_THEN);
      thenast->setText("then");
      thenast->setLine($2->getLine());
      thenast->setFirstChild($4);
      elseast = driver.newASTNode();
      elseast->setType(TK_K_IF_ELSE);
      elseast->setText("else");
      elseast->setLine($2->getLine());
      thenast->setNextSibling(elseast);
      stmt->getFirstChild()->setNextSibling(thenast);
    }
    /* Handle Iterator list (at least one value) */
    doast = $3;
    iter  = $1->getNextSibling();
    $3->setFirstChild($1);
    $1->setNextSibling(stmt);
    /* Du deuxieme au dernier faire */
    while (iter)
    {
      Yml::Core::AST* nparast = driver.newASTNode();
      Yml::Core::AST* ndoast = driver.newASTNode();

      nparast->setType(TK_K_PAR);
      nparast->setText("par");
      nparast->setFirstChild(ndoast);
      nparast->setLine($3->getLine()); 
      ndoast->setType(TK_K_DO);
      ndoast->setText("do");
      ndoast->setFirstChild(iter);
      ndoast->setLine($3->getLine());
      iter = iter->getNextSibling();
      ndoast->getFirstChild()->setNextSibling(stmt);
      doast->getFirstChild()->setNextSibling(nparast);
      doast = ndoast; 
    }
    $$=$3;
  }
;

  /* Quite hard rule because we reorder nodes from the instruction 
   * The do_cond_exclude_list is transformed into an if with an and between all expression
   * The do_cond_list is translated into imbricate par do statement with one 
   * iterator per level We first compute the if with statement list and then we does 
   * the expansion of the iterators 
   */
seqdo_block:
  do_cond_iterator_list do_cond_exclude_list TK_K_DO stmt_list TK_K_DO_END 
  {
   /* $1 -> iterators 
      $2 -> exclude 
      $3 -> do 
      $4 -> stmts 
    */
    
    /* Handle exclude list */ 
    Yml::Core::AST* stmt  = $4;
    Yml::Core::AST* doast = 0;
    Yml::Core::AST* iter  = 0;
    if ($2 != 0)
    {
      /* Create the if stmt arround $4 */
      Yml::Core::AST * tmp = $2;
      Yml::Core::AST * thenast = 0;
      Yml::Core::AST * elseast = 0;
      Yml::Core::AST * left = 0;
      Yml::Core::AST * andast = 0;
      Yml::Core::AST * funcast = 0;
      stmt = driver.newASTNode();
      stmt->setType(TK_K_IF);
      stmt->setText("if");
      stmt->setLine($2->getLine());
      stmt->setFirstChild(tmp);
      while (tmp->getNextSibling())
      {
        left = tmp;
        tmp = tmp->getNextSibling();
        left->setNextSibling(0);
        left = stmt->getFirstChild();
        funcast = driver.newASTNode();
        funcast->setType(TK_FUNC);
        funcast->setText("function");
        funcast->setLine(stmt->getLine());
        andast = driver.newASTNode();
        funcast->setFirstChild(driver.newASTNode());
        funcast->getFirstChild()->setText("");
        funcast->getFirstChild()->setType(TK_IDENTIFIER);
        funcast->getFirstChild()->setLine(stmt->getLine());
        funcast->getFirstChild()->setFirstChild(andast);
        andast->setType(TK_IDENTIFIER);
        andast->setText("operator_and");
        funcast->getFirstChild()->setNextSibling(left);
        left->setNextSibling(tmp);
        stmt->setFirstChild(funcast);
      }
      thenast = driver.newASTNode();
      thenast->setType(TK_K_IF_THEN);
      thenast->setText("then");
      thenast->setLine($2->getLine());
      thenast->setFirstChild($4);
      elseast = driver.newASTNode();
      elseast->setType(TK_K_IF_ELSE);
      elseast->setText("else");
      elseast->setLine($2->getLine());
      thenast->setNextSibling(elseast);
      stmt->getFirstChild()->setNextSibling(thenast);
    }
    /* Handle Iterator list (at least one value) */
    doast = $3;
    iter  = $1->getNextSibling();
    $3->setFirstChild($1);
    $1->setNextSibling(stmt);
    /* Du deuxieme au dernier faire */
    while (iter)
    {
      Yml::Core::AST* nparast = driver.newASTNode();
      Yml::Core::AST* ndoast = driver.newASTNode();

      nparast->setType(TK_K_SEQ);
      nparast->setText("seq");
      nparast->setFirstChild(ndoast);
      nparast->setLine($3->getLine()); 
      ndoast->setType(TK_K_DO);
      ndoast->setText("do");
      ndoast->setFirstChild(iter);
      ndoast->setLine($3->getLine());
      iter = iter->getNextSibling();
      ndoast->getFirstChild()->setNextSibling(stmt);
      doast->getFirstChild()->setNextSibling(nparast);
      doast = ndoast; 
    }
    $$=$3;
  }
;

do_cond_iterator:
  '(' assign_stmt ';' rvalue ')' { 
                                    $$= driver.newASTNode(); 
                                    $$->setType(TK_K_DO_COND_ITERATOR); 
                                    $$->setText("iterator");   
                                    $$->setLine($2->getLine());
                                    $$->setFirstChild($2);
                                    $2->setNextSibling($4); 
                                 }
;
do_cond_iterator_list:
  do_cond_iterator do_cond_iterator_list2 {$$=$1 ; $1->setNextSibling($2);}

do_cond_iterator_list2: 
  /* EMPTY */ {$$ = 0; }
|
  do_cond_iterator do_cond_iterator_list2 { $$=$1 ; $1->setNextSibling($2); }
;

do_cond_exclude_list:
  /* EMPTY */ {$$ = 0;}
|
  do_cond_exclude do_cond_exclude_list2 {$1->setNextSibling($2) ; $$=$1;}
;

do_cond_exclude_list2:
  /* EMPTY */ {$$ = 0;}
|
  do_cond_exclude do_cond_exclude_list2 {$1->setNextSibling($2) ; $$=$1;}                   
;

do_cond_exclude:
  '{' rvalue '}'  { 
                    /* 
                    $$ = driver.newASTNode(); 
                    $$->setType(TK_K_DO_COND_EXCLUDE); 
                    $$->setLine($2->getLine());
                    $$->setText("exclude");
                    $$->setFirstChild($2);
                    */ 
                    $$ = $2;
                  }
;
  /* if then else construction */
if_stmt:
  TK_K_IF rvalue TK_K_IF_THEN stmt_list if_else TK_K_IF_END {$1->setFirstChild($2); 
                                                              $2->setNextSibling($3); 
                                                              $3->setFirstChild($4);
                                                              $3->setNextSibling($5);
                                                              $$ = $1;
                                                              
                                                              }
;

if_else:
  /* EMPTY */ {$$ = driver.newASTNode(); $$->setType(TK_K_IF_ELSE); $$->setText("else"); }
|
  TK_K_IF_ELSE stmt_list { $1->setFirstChild($2); $$=$1;}
;


param_list:
  rvalue param_list2 { $$=$1; $1->setNextSibling($2);}
;
param_list2:
  /* EMPTY */  {$$=0l;}
|
  ',' rvalue param_list2 { $$=$2; $2->setNextSibling($3);}
;
  
  /* Left hand side and every where we are interested in a single value */
lvalue: 
  TK_IDENTIFIER range_list {$$ = $1; $1->setFirstChild($2);}
;

lvalue_list: 
  /* EMPTY */ {$$ = 0;}
|
  ',' lvalue lvalue_list {$2->setNextSibling($3); $$ = $2;}
;
/** Handle Range information */ 
range_list: 
  /* empty */ {$$=0 ; }
|
  range range_list {$1->setNextSibling($2); $$=$1;}
;
range:
  '[' rvalue ']' {$$ = $2;}
;

  /* Right hand side of an assignment */ 
rvalue:
  rvalue TK_OP_BOOL_AND rvalue_equality { $$=$2 ; $2->getFirstChild()->setNextSibling($1) ; $1->setNextSibling($3);}
| 
  rvalue TK_OP_BOOL_OR rvalue_equality { $$=$2 ; $2->getFirstChild()->setNextSibling($1) ; $1->setNextSibling($3); }
|
  rvalue TK_OP_BOOL_XOR rvalue_equality { $$=$2 ; $2->getFirstChild()->setNextSibling($1) ; $1->setNextSibling($3); }
|
  rvalue_equality {$$=$1;}
;

rvalue_equality:
  rvalue_equality TK_OP_COMP_EQ rvalue_comp { $$=$2 ; $2->getFirstChild()->setNextSibling($1) ; $1->setNextSibling($3);}
|
  rvalue_equality TK_OP_COMP_NEQ rvalue_comp { $$=$2 ; $2->getFirstChild()->setNextSibling($1) ; $1->setNextSibling($3);}
|
  rvalue_comp {$$=$1;}
;

rvalue_comp:
  rvalue_comp TK_OP_COMP_LT rvalue_add { $$=$2 ; $2->getFirstChild()->setNextSibling($1) ; $1->setNextSibling($3);}
|
  rvalue_comp TK_OP_COMP_LE rvalue_add { $$=$2 ; $2->getFirstChild()->setNextSibling($1) ; $1->setNextSibling($3);}
|
  rvalue_comp TK_OP_COMP_GE rvalue_add { $$=$2 ; $2->getFirstChild()->setNextSibling($1) ; $1->setNextSibling($3);}
|
  rvalue_comp TK_OP_COMP_GT rvalue_add { $$=$2 ; $2->getFirstChild()->setNextSibling($1) ; $1->setNextSibling($3);}
|
  rvalue_add {$$=$1;}
;

rvalue_add:
  rvalue_add TK_OP_ADD rvalue_mul { $$=$2 ; $2->getFirstChild()->setNextSibling($1) ; $1->setNextSibling($3);} 
|
  rvalue_add TK_OP_MIN rvalue_mul { $$=$2 ; $2->getFirstChild()->setNextSibling($1) ; $1->setNextSibling($3);}
|
  rvalue_mul {$$=$1;}
;

rvalue_mul:
  rvalue_mul TK_OP_MUL rvalue_primary {$$=$2 ; $2->getFirstChild()->setNextSibling($1); $1->setNextSibling($3);}
|
  rvalue_mul TK_OP_DIV rvalue_primary {$$=$2 ; $2->getFirstChild()->setNextSibling($1); $1->setNextSibling($3);}
| 
  rvalue_mul TK_OP_MOD rvalue_primary {$$=$2 ; $2->getFirstChild()->setNextSibling($1); $1->setNextSibling($3);}
| 
  rvalue_primary {$$=$1;}
;

rvalue_primary:
  TK_IDENTIFIER '(' rvalue_list ')' {
                                     $$ = driver.newASTNode() ; 
                                     $$->setType(TK_FUNC);
                                     $$->setText("function");
                                     $$->setLine($1->getLine());
                                     $$->setFirstChild(driver.newASTNode());
                                     $$->getFirstChild()->setType(TK_IDENTIFIER);
                                     $$->getFirstChild()->setText("");
                                     $$->getFirstChild()->setLine($1->getLine());
                                     $$->getFirstChild()->setFirstChild($1);
                                     $$->getFirstChild()->setNextSibling($3);
                                    } 
|
  TK_IDENTIFIER TK_K_NAMESPACE TK_IDENTIFIER '(' rvalue_list ')' {
                                                                $$ = driver.newASTNode();
                                                                $$->setType(TK_FUNC);
                                                                $$->setText("function");
                                                                $$->setLine($1->getLine());
                                                                $$->setFirstChild($1);
                                                                $1->setFirstChild($3);
                                                                $1->setNextSibling($5);
                                                               }
|
  '(' rvalue ')' {$$=$2;}
|
  TK_OP_BOOL_NOT rvalue_primary {$$=$1;$1->getFirstChild()->setNextSibling($2);}
|
  TK_OP_MIN rvalue_primary {$$=$1; $1->getFirstChild()->setNextSibling($2);}
|
  TK_INTEGER {$$=$1;}
|
  TK_REAL {$$=$1;}
|
  TK_STRING {$$=$1;}
|
  lvalue {$$ = $1;}
;      

rvalue_list: 
  /* EMPTY */ {$$=0;}
|
  rvalue rvalue_list2 { $$=$1;$1->setNextSibling($2);}
;

rvalue_list2:
  /* EMPTY */ {$$=0;}
|
  ',' rvalue rvalue_list2 {$$=$2;$2->setNextSibling($3);}
;

%%
void yy::YvetteParser::error(const yy::YvetteParser::location_type& l, const std::string& m)
{
	driver.error(l, m);
}
