/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/
/**
 * @file
 * @brief Class YvetteParserDriver function definition 
 *
 * 2005-12-27
 * Olivier Delannoy
 */
#include "YvetteParserDriver.hh"
#ifndef HAVE_INLINE
#define inline
#include "YvetteParserDriver.icc"
#undef inline
#endif 
#include "YvetteParser.hh"
namespace Yml 
{
    namespace Core
    {
        YvetteParserDriver::YvetteParserDriver(bool traceScanner, bool traceParser)
            : mAST(0), 
	      mTraceScanner(traceScanner), 
              mTraceParser(traceParser), 
              mErrorCount(0)
            
        {
            
        }
        YvetteParserDriver::~YvetteParserDriver(void)
        {
            std::vector<AST*>::iterator iter = mASTNodes.begin();
            const std::vector<AST*>::iterator end = mASTNodes.end();
            for(;iter != end ; ++iter) 
                delete *iter;
        }
        
        AST* YvetteParserDriver::parse(const string& input)
        {
            mAST = newASTNode();
            mAST->setType(0);
            mAST->setText("root");
            mErrorCount = 0;
            mErrors.clear();
            beginScan(input.c_str(), input.size());
            yy::YvetteParser parser(*this);
            parser.set_debug_level(mTraceParser);
            parser.parse();
            endScan();
            if (mErrorCount == 0)
                return mAST;
            return 0;
        }
        AST* YvetteParserDriver::parse(const char* input, uint32 length)
        {
            mAST = newASTNode();
            mAST->setType(0);
            mAST->setText("root");
            mErrorCount = 0;
            mErrors.clear();
            beginScan(input, length);
            yy::YvetteParser parser(*this);
            parser.set_debug_level(mTraceParser);
            parser.parse();
            endScan();
            if (mErrorCount == 0)
                return mAST;
            return 0;
        }
        void YvetteParserDriver::error(const yy::location& l, const std::string& m)
        {
            mErrors.push_back(ParserError(m, l.begin.line, l.begin.column));
            ++mErrorCount;
        }
        void YvetteParserDriver::error(const std::string& m)
        {
            ++mErrorCount;
            mErrors.push_back(ParserError(m));
        }
    }
}
