/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/
/**
 * @file
 * @brief Class YvetteScanner generator
 *  
 * 2005-12-27
 * Olivier Delannoy
 */

%{
#include "YvetteParser.hh"
#include "YvetteParserDriver.hh"
#define MAX_STR_CONST 1024
#define CREATE_AST(res, type, value)\
{\
  Yml::Core::AST *tmp = driver.newASTNode();\
  tmp->setLine(yylloc->begin.line);\
  tmp->setType((type));\
  tmp->setText((value));\
  (res)= tmp;\
}

#define CREATE_AST_OP(res, type, value)\
{\
  Yml::Core::AST *tmp = driver.newASTNode();\
  Yml::Core::AST *tmp2 = driver.newASTNode();\
  Yml::Core::AST *tmp3 = driver.newASTNode();\
  tmp->setText("function");\
  tmp->setType(TK_FUNC);\
  tmp->setLine(yylloc->begin.line);\
  tmp->setFirstChild(tmp2);\
  tmp2->setText("");\
  tmp2->setType(TK_IDENTIFIER);\
  tmp2->setLine(tmp->getLine());\
  tmp2->setFirstChild(tmp3);\
  tmp3->setType(TK_IDENTIFIER);\
  tmp3->setLine(tmp->getLine());\
  tmp3->setText(value);\
  (res)=tmp;\
}
%}
%option nostdinit  nounput batch debug
%{
#define YY_USER_ACTION yylloc->columns(yyleng);
%}
%x str
DIGIT [0-9]
ALPHA [a-zA-Z]

%%
%{
  yylloc->step();
%}
  /* Handle strings */
    char my_string_buf[MAX_STR_CONST];
    char *my_string_buf_ptr;
    
  /* Scheduling policy */
"compute" CREATE_AST(*yylval, TK_K_COMPUTE, yytext);                                          return TK_K_COMPUTE;
"migrate" CREATE_AST(*yylval, TK_K_MIGRATE, yytext);                                          return TK_K_MIGRATE;

"par"       CREATE_AST(*yylval, TK_K_PAR, yytext);                                            return TK_K_PAR;
"endpar"    *yylval = 0;                                                                      return TK_K_PAR_END;
"//"        CREATE_AST(*yylval, TK_K_PAR_SEP, yytext);                                        return TK_K_PAR_SEP;
"seq"       CREATE_AST(*yylval, TK_K_SEQ, yytext);                                            return TK_K_SEQ;
"do"        CREATE_AST(*yylval, TK_K_DO, yytext);                                             return TK_K_DO;
"enddo"     *yylval = 0;                                                                      return TK_K_DO_END;
"if"        CREATE_AST(*yylval, TK_K_IF, yytext);                                             return TK_K_IF;
"endif"     *yylval = 0;                                                                      return TK_K_IF_END;
"then"      CREATE_AST(*yylval, TK_K_IF_THEN, yytext);                                        return TK_K_IF_THEN;
"else"      CREATE_AST(*yylval, TK_K_IF_ELSE, yytext);                                        return TK_K_IF_ELSE;
"notify"    CREATE_AST(*yylval, TK_K_NOTIFY, yytext);                                         return TK_K_NOTIFY;
"wait"      CREATE_AST(*yylval, TK_K_WAIT, yytext);                                           return TK_K_WAIT;
"Event"     CREATE_AST(*yylval, TK_K_EVENT, yytext);                                          return TK_K_EVENT;

  /* Operator */
\+          CREATE_AST_OP(*yylval, TK_OP_ADD,      "operator_add");                           return TK_OP_ADD;
-           CREATE_AST_OP(*yylval, TK_OP_MIN,      "operator_sub");                           return TK_OP_MIN;
\*          CREATE_AST_OP(*yylval, TK_OP_MUL,      "operator_mul");                           return TK_OP_MUL;
\/          CREATE_AST_OP(*yylval, TK_OP_DIV,      "operator_div");                           return TK_OP_DIV;
\%          CREATE_AST_OP(*yylval, TK_OP_MOD,      "operator_mod");                           return TK_OP_MOD;

"and"       CREATE_AST_OP(*yylval, TK_OP_BOOL_AND, "operator_and");                           return TK_OP_BOOL_AND;
"or"        CREATE_AST_OP(*yylval, TK_OP_BOOL_OR,  "operator_or");                            return TK_OP_BOOL_OR;                 
"xor"       CREATE_AST_OP(*yylval, TK_OP_BOOL_XOR, "operator_xor");                           return TK_OP_BOOL_XOR;
"not"       CREATE_AST_OP(*yylval, TK_OP_BOOL_NOT, "operator_not");                           return TK_OP_BOOL_NOT;

"eq"        CREATE_AST_OP(*yylval, TK_OP_COMP_EQ,  "operator_eq");                            return TK_OP_COMP_EQ;
"neq"       CREATE_AST_OP(*yylval, TK_OP_COMP_NEQ, "operator_neq");                           return TK_OP_COMP_NEQ;
"le"        CREATE_AST_OP(*yylval, TK_OP_COMP_LE,  "operator_le");                            return TK_OP_COMP_LE;
"ge"        CREATE_AST_OP(*yylval, TK_OP_COMP_GE,  "operator_ge");                            return TK_OP_COMP_GE;
"lt"        CREATE_AST_OP(*yylval, TK_OP_COMP_LT,  "operator_lt");                            return TK_OP_COMP_LT;
"gt"        CREATE_AST_OP(*yylval, TK_OP_COMP_GT,  "operator_gt");                            return TK_OP_COMP_GT;

  /* Assignment or namespace */
":="        CREATE_AST(*yylval, TK_K_ASSIGN, yytext);                                         return TK_K_ASSIGN;
"::"        *yylval =  0;                                                                     return TK_K_NAMESPACE;

  /* Simplificate the life of programmers */
"true"      CREATE_AST(*yylval, TK_INTEGER, "1");                                             return TK_INTEGER;
"false"     CREATE_AST(*yylval, TK_INTEGER, "0");                                             return TK_INTEGER;

  /* Exception Handling keyword (not yet available)  
  "Exception"   CREATE_AST(*yylval, TK_K_EX_EXCEPTION, yytext);       return TK_K_EX_EXCEPTION; 
  "try"         CREATE_AST(*yylval, TK_K_EX_TRY, yytext);             return TK_K_EX_TRY;
  "endtry"      CREATE_AST(*yylval, TK_K_EX_TRY_END, yytext);         return TK_K_EX_TRY_END;
  "catch"       CREATE_AST(*yylval, TK_K_EX_CATCH, yytext);           return TK_K_EX_CATCH;
  "endcatch"    CREATE_AST(*yylval, TK_K_EX_CATCH_END, yytext);       return TK_K_EX_CATCH_END;
  "finally"     CREATE_AST(*yylval, TK_K_EX_FINALLY, yytext);         return TK_K_EX_FINALLY;
  "endfinally"  CREATE_AST(*yylval, TK_K_EX_FINALLY_END, yytext);     return TK_K_EX_FINALLY_END;
  */
{ALPHA}[a-zA-Z0-9_]*                          CREATE_AST(*yylval, TK_IDENTIFIER, yytext);    return TK_IDENTIFIER;

  /* Integers */
[1-9][0-9]*i?|0i?                              CREATE_AST(*yylval, TK_INTEGER, yytext);    return TK_INTEGER;

  /* Reals */
(0|[1-9][0-9]*)r                                  CREATE_AST(*yylval, TK_REAL, yytext);     return TK_REAL;  
(0|[1-9][0-9]*)\.[0-9]+r?                         CREATE_AST(*yylval, TK_REAL, yytext);     return TK_REAL;
(0|[1-9][0-9]*)(\.[0-9]+)?(e|E)-?([0-9]+)  CREATE_AST(*yylval, TK_REAL, yytext);     return TK_REAL;      
  /* Strings */
                               
\"                    my_string_buf_ptr = my_string_buf;   BEGIN(str); 
<str>\"               {        
                        BEGIN(INITIAL);
                        *my_string_buf_ptr = '\0';
                        CREATE_AST(*yylval, TK_STRING, my_string_buf);                        return TK_STRING;
                      }
<str>\n               {
                        /* String not terminated at the end of the line, it's an error */
                      }
<str>\\[0-7]{1,3}     { 
                        int result; 
                        (void) sscanf( yytext + 1, "%o", &result );
                        if ( result > 0xff )  
                        {
                          /* Range error */
                        }
                        *my_string_buf_ptr++ = result;
                      }
<str>\\[0-9]+         {
                        /* Not an octal value number of octal value are between 0-7 */
                      }
<str>\\n              *my_string_buf_ptr++ = '\n'; 
<str>\\t              *my_string_buf_ptr++ = '\t'; 
<str>\\r              *my_string_buf_ptr++ = '\r';
<str>\\b              *my_string_buf_ptr++ = '\b';
<str>\\f              *my_string_buf_ptr++ = '\f';
<str>\\(.|\n)         *my_string_buf_ptr++ = yytext[1]; 
<str>[^\\\n\"]+       {
                          char *yptr = yytext;
                        while (*yptr)
                          *my_string_buf_ptr++ = *yptr++;
                      }

<<EOF>>     *yylval = 0;                                                                       return TK_EOF; 
	/* comments */
#(.*)$       yylloc->step();
[ \t\r]+     yylloc->step();
[\n]+        yylloc->lines(yyleng), yylloc->step();   
[;(){}[\],]   return yytext[0];
.            driver.error(*yylloc, std::string("invalid caracter >") + yytext[0] + "<");

%%
namespace Yml
{
	namespace Core 
	{
		void YvetteParserDriver::beginScan(const char* tmp, uint32 length)
		{
            yy_flex_debug = mTraceScanner;
            mBuffer =(void*) yy_scan_bytes(tmp, length); 
		}
		void YvetteParserDriver::endScan(void)
		{
            yy_delete_buffer((YY_BUFFER_STATE)mBuffer);
		}
	}
}

int yywrap()
{
    return 1;
}
