/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class YvetteSymbolTable declaration
 *  
 * 2005-06-28
 * Olivier Delannoy
 */
#ifndef YML_CORE_YVETTESYMBOLTABLE_HH
#define YML_CORE_YVETTESYMBOLTABLE_HH 1
#include "core.hh"
#include "YvetteObject.hh"

#include <vector>
#include <map>

namespace Yml {
    namespace Core {

        /**
         * @brief Store association between var names and values. 
         *
         * It's a context based symbol table based. It is easy to
         * access symbols in different context but it's only possible
         * to add a symbol in the current context. Newly created
         * context can overload variable, but the ancien value is no
         * more accessible, however, it's preserverd at the
         * destruction of the context. 
         */
        class YvetteSymbolTable
        {
        public:
            /**
             * Default Contstructor
             */
            YvetteSymbolTable(void);
            /**
             * Destructor
             */
            ~YvetteSymbolTable(void);
            
            /**
             * Create a new Context 
             */
            void pushContext();
            /** 
             * Destroy the current context
             */
            void popContext();
            
            /** 
             * Add a symbol to the table in its current context 
             * @param name The name of the symbol 
             * @param object the value associated to the symbol 
             * 
             * @return true if the symbol was added false otherwise.
             * This function always succeed except if the symbol
             * corresponds to an event or refer to it.
             */
            bool addSymbol(const string& name, const YvetteObject& object);
            
            /** 
             * Retrieve symbol information  
             *
             * @param name the name of the symbol 
             *
             * @param currentContextOnly if true the
             * symbol will be search for only in the current context
             * otherwise all context will be check.
             *
             * @return true if the symbol exists false otherwise. 
             */
            const YvetteObject& getSymbol(const string& name, bool currentContextOnly = false);
        protected:
    
        private:
            /** Typename for the context */
            typedef std::map<string, YvetteObject> Context;
            /** Typename for a collection of context */
            typedef std::vector<Context* > Symbols;
            
            /** 
             * Store all symbols 
             */
            Symbols mSymbols;
        };
    }
}
#ifdef HAVE_INLINE
#include "YvetteSymbolTable.icc"
#endif
#endif

