/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class YvetteTaskGenerator declaration
 *  
 * 2005-06-19
 * Olivier Delannoy
 */
#ifndef YML_CORE_YVETTETASKGENERATOR_HH
#define YML_CORE_YVETTETASKGENERATOR_HH 1
#include "core.hh"
#include "YvetteContext.hh" 
namespace Yml {
    namespace Core {
        class YvetteCompiler;
        class YvetteObject;
        class AST;
        class PreCondition;
        
        /**
         * @brief The task generator is used to create task call in the application. 
         * 
         * It analyses the Yvette program tree represented by an AST
         * and evaluates it in order to generate an application. At
         * the end of the process the whole task set of the
         * application is available. 
         */
        class YvetteTaskGenerator
        {
        public:
            /**
             * Default Contstructor
             */
            YvetteTaskGenerator(YvetteCompiler* compiler);
            /**
             * Destructor
             */
            ~YvetteTaskGenerator(void);
            
            /**
             * Create all tasks corresponding to a tree 
             * The tree must begin with a node named "root" 
             */
            bool generate(AST* tree, YvetteContext& context);
            

        protected:
            bool visitAssignment(AST* tree);
            bool visitEventWait(AST* tree, YvetteContext& context);
            bool visitEventNotify(AST* tree, YvetteContext& context);
            bool visitComponentCall(AST* tree, uint32 schedulingPolicy, YvetteContext& context);
            bool visitConditional(AST* tree, YvetteContext& context);
            bool visitPar(AST* tree, YvetteContext& context);
            bool visitSeq(AST* tree, YvetteContext& context);
            /**
             * Descend an expression and return the YvetteObject corresponding 
             */
            YvetteObject visitRValue(AST* tree);
            /** 
             * Descend an identifier and return a pointer to the corresponding string 
             */
            bool visitLValue(AST* tree, string& result);
            
            /**
             * Descend an event expression and return a pointer to it 
             */
            PreCondition* visitEventCondition(AST* tree);
            
            
        private:
            YvetteCompiler *mCompiler;
        };
    }
}
#ifdef HAVE_INLINE
#include "YvetteTaskGenerator.icc"
#endif
#endif

