/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/
/**
 * @file
 * @brief Class Pool declaration
 *  
 * 2006-04-10
 * Olivier DELANNOY
 */
#ifndef YML_POOL_HH
#define YML_DR_POOL_HH 1
#include "dr_namespace.hh"
#include <list> 
namespace Yml {
    namespace DR
    {
        class DRServerThread;
        
        /**
         * @brief Pool of thread class
         *
         * There is a strange problem with the standard list class. By
         * creating our own class we will try to better understand this
         * strange behaviour. It's a temporary solution.
         */
        class Pool
        {
            typedef std::list<DRServerThread*> Threads;
            
        public:
            /**
             * Default Contstructor
             */
            Pool(void);
            /**
             * Destructor
             */
            ~Pool(void);
            /** 
             * Get the size of the pool 
             */
            uint32 size(void) const;
            /** 
             * Register a new Server Thread 
             * 
             */
            void push_back(DRServerThread* obj);
            /**
             * Clean finished server thread 
             */
            void clean(void);
        protected:
    
        private:
            /** Store all threads */
            Threads mThreads;
            /** Store the current size of the pool */ 
            uint32 mSize;
            
            /* Disabled operation */
            Pool(const Pool& obj);
            /* Disabled operation */
            Pool& operator=(const Pool& obj);
        };
    }
}
#endif
