/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/

/**
 * @file
 * @brief Class Scheduler declaration
 *  
 * 2005-07-18
 * Olivier Delannoy
 */
#ifndef YML_SCHEDULER_HH
#define YML_SCHEDULER_HH 1
#include "scheduler_namespace.hh"
#include <Yml/Core/YvetteApplication.hh> 
#include <Yml/Core/PluginManager.hh> 

#include <Application.hh>
namespace Yml {
    /** 
     * @brief Contains all classes specifics to the scheduler 
     * 
     * All class related to the scheduler are grouped in this
     * namespace. 
     */
    namespace Scheduler {

        /**
         * @brief Scheduler main class 
         * 
         * This class correspond to the "main" function of the scheduler 
         */
        class Scheduler : public Util::Application 
        {
        public:
            /**
             * Default Contstructor
             */
            Scheduler(Util::ApplicationAdaptor* adaptor, int argc, char** argv);
            /**
             * Destructor
             */
            ~Scheduler(void);
        protected:
            string usageQuick(void) const;
            string usageParameters(void) const;
            string usageOptions(void) const;
            void configureCommandLine(void);
            string configDir(void) const;
            string loggersDir(void) const;
            string pluginDir(void) const;
            uint32 run(void);
        private:
            Yml::Core::PluginManager mManager;
            /** Store the path  to the query */
            string mApplicationFile;
            /** Store the working dir for this execution*/
            string mWorkingDir;
            /** Store the name of the input pack if provided */    
            string mInputParameterPackName;
            /** Store the name of the output pack if provided */
            string mOutputParameterPackName;
            /** Store wether cleanup is enabled or not */ 
            bool mDoCleanup;
            
        };
    }
}
#endif

