/* -*- mode: c; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/
/**
 * @file
 * @brief Default Worker header file drclient
 *  
 * 2006-04-16
 * Olivier Delannoy
 */
#ifndef YML_WORKER_DRCLIENT_H
#define YML_WORKER_DRCLIENT_H 1
#ifdef __cplusplus
extern "C" {
#endif 
#include "worker_private.h"
typedef void* drclient_t;
/**
 * Connect to a data repository server 
 * 
 * @param client A pointer to the handler object
 * 
 * @param host the name of the host to connect to 
 * 
 * @param port the port number 
 * 
 * @return 0 on success -errno on error
 */
int drclient_connect(drclient_t* client, char* host, unsigned short port);
/**
 * Disconnect from a data repository server 
 *
 * @param client An handler to the data repository server.
 */
void drclient_disconnect(drclient_t client);
/**
 * Disconnect from a data repository server and connect to another one if needed 
 * 
 * @param client the handler object
 * 
 * @param host the name of the host to connect to 
 * 
 * @param port the port number 
 * 
 * @return 0 on success -errno on error
 */
int drclient_change_server(drclient_t client, char* host, unsigned short port);
/**
 * Download a file from the data repository server 
 *
 * @param client An handler to the data repository server.
 *
 * @param uri the remote path to the resource
 * 
 * @param local the local file to store the resource
 * 
 * @return 0 on success -errno on error
 */
int drclient_get(drclient_t client, char* uri, char* local);
/**
 * Download a file from the data repository server 
 *
 * @param client An handler to the data repository server.
 *
 * @param uri the remote path to the resource
 * 
 * @param data the local memory chunk which will contains the result 
 * 
 * @param dataSize the size of the resulting memoryChunk
 * 
 * @return 0 on success -errno on error
 */
int drclient_get_data(drclient_t client, char* uri, char** data, uint32* dataSize);
/**
 * Upload a file to the data repository server. 
 *
 * @param client An handler to the data repository server.
 *
 * @param uri the remote path to the resource  
 * 
 * @param local the local file to be used  
 * 
 * @return 0 on success -errno on error
 */
int drclient_put(drclient_t client, char* uri, char* local);
#ifdef __cplusplus
}
#endif 
#endif
