/* -*- mode: c; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/****************************************************************************
 * Copyright Olivier Delannoy, Laurent Choy 2005
 *
 * Olivier Delannoy   
 *   Olivier.Delannoy@gmail.com
 *   PRiSM Laboratory 
 *   Verailles University 
 *   45 avenue des Etats Unis 
 *   78035 Versailles Cedex FRANCE 
 *
 * Laurent Choy 
 *   Choy.Laurent@gmail.com
 *   INRIA Futurs
 *   Parc Club Orsay Universit                      
 *   ZAC des vignes                      
 *   2-4, rue Jacques Monod                      
 *   91893 Orsay Cedex France
 *
 *
 *
 * This software is a computer program whose purpose is to [describe
 * functionalities and technical features of your software].
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 ****************************************************************************/
/**
 * @file
 * @brief Default Worker header file pack
 *  
 * 2006-04-14
 * Olivier Delannoy
 */
#ifndef YML_WORKER_PACK_H
#define YML_WORKER_PACK_H 1
#ifdef __cplusplus
extern "C" {
#endif 
#include "worker_private.h"
/** 
 * @brief Pack files together in an archive. 
 * 
 * This is a pure c / posix implementation of pack files (similar to tar)
 */
struct pack;
typedef struct pack* pack_t;
/** 
 * Open a pack from a file 
 * @param pack the result pack handler 
 * @param file the name of the file 
 * @return 0 on success -errno on error 
 */
int pack_open(pack_t* pack, char* file);
/**
 * Close a pack 
 * @param pack the pack to close 
 */
void pack_close(pack_t pack);
/**
 * Extract all pack entries 
 * @param pack the pack handler 
 * @param dir the destination folder 
 * @return 0 on success -errno on error 
 */
int pack_extract(pack_t pack, char* dir);
/**
 * Add a new pack entry from memory 
 * 
 * @param pack the pack handler 
 * @param name the name of the file 
 * @param data the byte composing the entry 
 * @param size the size of the data 
 * @return 0 on success -errno on error 
 */
int pack_add_entry(pack_t pack, char* name, char* data, uint32 size);
/**
 * Add a new pack entry from file 
 * 
 * @param pack the pack handler 
 * @param name the name of the entry 
 * @param file the name of the file 
 * @return 0 on success -errno on error 
 */ 
int pack_add_file(pack_t pack, char* name, char* filename);
/** 
 * Add a new pack entry from file 
 * 
 * @param pack the pack handler 
 * @param name the name of the entry
 * @param fd the file descriptor 
 */
int pack_add_ffile(pack_t pack, char* name, int fd);
#ifdef __cplusplus
}
#endif 
#endif
/****************************************************************************/
